/*  Gaby
 *  Copyright (C) 1998 Frederic Peters
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gaby.h"
#include <sys/stat.h>

struct _type_form_widget {
	gpointer *widget;
	int type;
	int flags;
};

typedef struct _type_form_widget tformwidget;

#ifdef TESTBASE
gboolean sort_by_lastname=TRUE;
tformwidget ffields[MAX_LABELS];
#else
extern gboolean sort_by_lastname;
extern tformwidget ffields[MAX_LABELS];
#endif
gboolean readthisline ( GString *st, record *ent );
gint sort_c=0;
gint nbook;

void base_init()
{
//	all_records = g_list_alloc();
//	all_bases = g_list_alloc();
}

gint* bases_vsblt(gint *az)
{
	gint i;
	book *bo;
	az = g_malloc(sizeof(gint) * nbook);
	all_bases = g_list_first(all_bases);
	for ( i=0;i<nbook;i++) {
		bo = all_bases->data;
		if ( bo->deleted ) { az[i] = FALSE; }
		else { az[i] = bo->shown; }
		if ( all_bases->next != NULL )
			all_bases = g_list_next(all_bases);
	}
	return az;
}

gint record_goto(gint nb)
{
	record *tmpent;
	gint i=0;
	gint lastshown=0;
	gboolean foo=FALSE,bar=FALSE;
	gint *az=NULL;
	
	if ( all_records == NULL ) { return (-2); }
	if ( all_records->data == NULL ) { return (-2); }

	az = bases_vsblt(az);
	all_records = g_list_first(all_records);
	i=0;
	while ( i <= nb ) {
		tmpent = all_records->data;
		if ( tmpent->shown == TRUE && az[tmpent->frombook-1] == TRUE) {
			lastshown = i;
			i++;
			bar = TRUE; /* at least one visible record */
		}
		if ( all_records->next == NULL ) {
			if ( !bar ) {
				g_free(az);
				return (-2);
			}
			if ( lastshown == nb ) {
				foo = TRUE;
				break;
			}
			nb = lastshown;
			record_goto(nb);
			g_free(az);
			return nb;
		} else {
			all_records = g_list_next(all_records);
		}
	}
	
	if ( !foo ) {
		all_records = g_list_previous(all_records);
	}
	g_free(az);
	
	return nb;
}

gint record_goto_str(GString *val, gint field, gint startfrom)
{
	record *tmpent;
	gint i=0;
	gboolean dontstop;
	gint *az=NULL;
	
	if ( all_records == NULL ) { return (-2); }
	if ( all_records->data == NULL ) { return (-2); }

	az = bases_vsblt(az);
	all_records = g_list_first(all_records);
	while ( i <= startfrom ) {
		tmpent = all_records->data;
		if ( tmpent->shown == TRUE && az[tmpent->frombook-1] == TRUE) {
			i++;
		}
		if ( all_records->next == NULL ) {
			break;
		} else {
			all_records = g_list_next(all_records);
		}
	}

	dontstop = TRUE;
	i=startfrom;
	while ( dontstop ) {
		tmpent = all_records->data;
		i++;
		if ( tmpent->shown == TRUE && az[tmpent->frombook-1] == TRUE) {
			if ( strcmp(tmpent->info[field]->str, val->str)==0 ) {
				g_free(az);
				return i;
			}
		}
		
		if ( all_records->next == NULL ) {
			dontstop = FALSE;
		} else {
			all_records = g_list_next(all_records);
		}
	}
	
	g_free(az);
	return -2;
}

static int compar_string ( gpointer *a, gpointer *b)
{
	int i;
	gpointer *c=a;
	gpointer *d=b;
	GString *sta, *stb;

	sta = *(c+1);
	stb = *(d+1);
	i = strcoll((char *)(sta->str), (char *)(stb->str));
	if ( i == 0 ) i = -1; /* this looks better */

	return i;
}

static int compar_numeric ( gpointer *a, gpointer *b)
{
	gpointer *c=a;
	gpointer *d=b;
	
	double *e=(double *)*(c+1);
	double *f=(double *)*(d+1);

	return ( ( *e > *f ) ? 1 : -1 );
}

static int compar_datetime ( gpointer *a, gpointer *b)
{
	gpointer *c=a;
	gpointer *d=b;
	
	time_t *e=(time_t *)*(c+1);
	time_t *f=(time_t *)*(d+1);

	return ( ( *e > *f ) ? 1 : -1 );
}

void date2time_t(time_t *time, GString *st)
{
	struct tm tim;
	
	tim.tm_hour = 0; tim.tm_min = 0; tim.tm_sec = 0;
	tim.tm_mday = 1; tim.tm_mon = 1; tim.tm_year = 1;
	
	if ( strrchr(st->str, '/' ) == NULL ) {
		tim.tm_year = atoi(st->str);
	} else {
		tim.tm_year = atoi(strrchr(st->str, '/' )+1);
		st = g_string_erase(st,st->len - \
			strlen(strrchr(st->str, '/' )), \
			strlen(strrchr(st->str, '/' )));
		if(strrchr( st->str,'/' ) == NULL) {
			tim.tm_mon = atoi(st->str);
		} else {
			tim.tm_mon = atoi( strrchr(st->str, '/' )+1);
			st = g_string_erase(st, \
				st->len - strlen(strrchr(st->str,'/')), \
				strlen(strrchr(st->str, '/' )));
			tim.tm_mday = atoi(st->str);
		}
	}
	*time = mktime(&tim);
}

void time2time_t(time_t *time, GString *st)
{
	struct tm tim;
	
	tim.tm_hour = atoi(st->str);
	tim.tm_min = atoi(strchr(st->str, ':' )+1);
	tim.tm_sec = atoi(strchr(strchr(st->str, ':'), ':')+1);
	*time = mktime(&tim);
}

gboolean base_sort(gint col)
{
	int i, j;
	int nbel;
	gpointer *ents;
	GString *st;
	int add_info=0;
	record *ent;
	double *value;
	time_t *time;
	
	if ( all_records == NULL ) { return TRUE; }
	
	sort_c = col;
	
	all_records = g_list_first(all_records);
	nbel=1;
	while ( all_records->next != NULL ) {
		all_records = g_list_next(all_records);
		nbel++;
	}
	ents = g_malloc(sizeof(ents)*2*nbel);
	all_records = g_list_first(all_records);
	for (i=0;i<nbel;i++) {
		ents[i*2] = all_records->data;
		if ( all_records->next != NULL) {
			all_records = g_list_next(all_records);
		}
	}
	all_records = g_list_first(all_records);
	
	switch ( ffields[sort_c].flags & 31 ) {
		case DATE:
		{
		add_info = 2;
		for (i=0;i<nbel;i++) {
			ent = ents[i*2];
			st = g_string_new(ent->info[sort_c]->str);
			time = g_malloc(sizeof(time_t));
			date2time_t(time, st);
			ents[i*2+1] = time;
			g_string_free(st, 1);
		}
		qsort(ents, (size_t)nbel, sizeof(gpointer)*2, \
					(int (*)())compar_datetime);
		} break;
		case TIME:
		{
		add_info = 2;
		for (i=0;i<nbel;i++) {
			ent = ents[i*2];
			st = g_string_new(ent->info[sort_c]->str);
			time = g_malloc(sizeof(time_t));
			time2time_t(time, st);
			ents[i*2+1] = time;
			g_string_free(st, 1);
		}
		qsort(ents, (size_t)nbel, sizeof(gpointer)*2, \
					(int (*)())compar_datetime);
		} break;
		case NUMERIC:
		{
		add_info = 2;
		for (i=0;i<nbel;i++) {
			ent = ents[i*2];
			st = g_string_new(ent->info[sort_c]->str);
			value = g_malloc(sizeof(double));
			*value = atof(st->str);
			ents[i*2+1] = value;
			g_string_free(st, 1);
		}
		qsort(ents, (size_t)nbel, sizeof(gpointer)*2, \
					(int (*)())compar_numeric);
		} break;
		case FULLNAME:
		default:
		{
		add_info = 1;
		for (i=0;i<nbel;i++) {
			ent = ents[i*2];
			st = g_string_new(ent->info[sort_c]->str);
			if ( (ffields[sort_c].flags & FULLNAME ) && \
					sort_by_lastname == TRUE ) {
				j = st->len;
				while ( st->str[j] != ' ' ) { j--; }
				for (;j>=0;j--){st=g_string_erase(st, j, 1); }
			}
			ents[i*2+1] = st;
		}
		qsort(ents, (size_t)nbel, sizeof(gpointer)*2, \
					(int (*)())compar_string);
		} break;
	}

	all_records = g_list_first(all_records);
	for (i=0;i<nbel;i++) {
		all_records->data = ents[i*2];
		if ( add_info == 1 ) { g_string_free(ents[i*2+1],1); }
		else { g_free(ents[i*2+1]); }
		if ( all_records->next != NULL) {
			all_records = g_list_next(all_records);
		}
	}
	
	g_free(ents);
	
	return TRUE;
}

double base_maths(int op, int field)
{
	gboolean ft=TRUE;
	gboolean dontstop=TRUE;
	record *ent;
	double result=0;
	int nbr=0;
	
	if ( all_records == NULL ) { return -1; }
	
	all_records = g_list_first(all_records);
	do {
		ent = all_records->data;
		if ( ent->shown == TRUE ) {
			nbr++;
			switch ( op ) {
				case 0: /* sum */
				case 1: /* average */
				{
				  result+=atof(ent->info[field]->str);
				} break;
				case 2: /* max */
				{
				  if ( ft==TRUE ) {
					result = atof(ent->info[field]->str);
					ft = FALSE;
				  } else {
					if(atof(ent->info[field]->str)>result)
						result = atof( \
							ent->info[field]->str);
				  }
				} break;
				case 3: /* min */
				{
				  if ( ft==TRUE ) {
					result = atof(ent->info[field]->str);
					ft = FALSE;
				  } else {
					if(atof(ent->info[field]->str)>result)
						result = atof( \
							ent->info[field]->str);
				  }
				} break;
			}
		}
		
		if ( all_records->next == NULL ) {
			dontstop = FALSE;
		} else {
			all_records = g_list_next(all_records);
		}
	} while ( dontstop == TRUE );

	switch ( op ) {
		case 1:  return (result/nbr); break;
		default: return result;
	}

}

gboolean readthisline ( GString *st, record *ent )
{
	int i;
	int row;
	int nbe=0;
	int foo;
	gchar c;
	row=0;
	if ( st->len < nlabels )
		return FALSE;
	if ( st->str[0] == '#' )
		return FALSE;
/* people want more and more entries (cell phone, state, ...) and the comma
 * delimited format is not the ideal one for this but since I don't want to
 * move to another one I have to find something to watch for which version a
 * book was done, so I count the commas and with this information I know what
 * entry is missing.
 * 
 * The format began with a total of 14 entries.
 * Changes are :
 *   - 0.0.5 - cell phone		(entry 10, total 15)
 *   - 0.1.2 - state			(entry  6, total 16)
 *
 */

/* Update : now that gaby != Gaby, the previous now only applies to gaby and 
 * not to other base using Gaby.
 */

	for (i=0;i<(st->len);i++) {
		if ( st->str[i] == ';' ) if ( st->str[i-1] != '\\' ) nbe++;
	}
	switch ( prog ) {
		case 1:
		{
			if ( nbe == 14 ) { /* before 0.0.5 */
				foo = 0;
				for (i=0; i<st->len; i++) {
					if ( st->str[i] == ';' ) {
						if ( st->str[i-1] != '\\' )
							foo++;
					}
					if ( foo == 10 ) { break; }
				}
				st = g_string_insert(st, i, ";");
				nbe++;
			}
			if ( nbe == 15 ) { /* before 0.1.2 */
				foo = 0;
				for (i=0; i<st->len; i++) {
					if ( st->str[i] == ';' ) {
						if ( st->str[i-1] != '\\' )
							foo++;
					}
					if ( foo == 6 ) { break; }
				}
				st = g_string_insert_c(st, i, ';');
			nbe++;
			}
		}
	}

	for (i=0;i<(st->len);i++) {
		ent->info[row] = g_string_new("");
		while ( st->str[i] != ';' && st->str[i] != '\n' ) {
			if ( st->str[i] == '\\' ) {
				i++;
				switch ( st->str[i] ) {
					case '\\': c = '\\'; break;
					case 'n' : c = '\n'; break;
					case 'r' : c = '\r'; break;
					case ';' : c = ';' ; break;
					case 't' : c = '\t'; break;
					default:   c = '?' ; break;
				}
				ent->info[row] = g_string_append_c( \
						ent->info[row], c );
			} else {
				if ( st->str[i] == '\r' ) {
					st->str[i] = '\n';
				}
				ent->info[row] = g_string_append_c( \
						ent->info[row], st->str[i] );
			}
			i++;
		}
		row++;
		if ( row > nlabels+1 ) {
			break;
//			return FALSE;
		}
	}
	
	for (;row <= nlabels;row++) {
		ent->info[row] = g_string_new("");
	}
	
	return TRUE;
}

void parsefilename(GString *fn)
{
	char *homedir;

	if ( fn->str[0] == '~' ) {
		homedir=getenv("HOME");
		fn = g_string_erase(fn, 0, 1);
		fn = g_string_prepend(fn, homedir);
	}
}

gboolean record_add (GString *st, gint fn)
{
	record *ent;
	ent = g_malloc((size_t)sizeof(record));
	if ( readthisline(st,ent) ) {
		ent->frombook = fn;
		ent->shown = TRUE;
		if ( all_records == NULL ) {
			all_records = g_list_alloc();
			all_records->data = (gpointer)ent;
		} else {
			all_records = g_list_last(all_records);
			all_records = g_list_append(all_records,(gpointer)ent );
		}
	} else {
		g_free(ent);
	}									
	
	return TRUE;
}

gboolean base_add (char *filename)
{
	/* todo : test if filename is not already loaded */
	FILE *fi;
	long eof, cur_pos;
	GString *st;
	book *bo;
	char *cfi;
	char c[1];
	GString *fn;

	struct stat m;
	
	fn = g_string_new("");
	fn = g_string_assign(fn, filename);
	parsefilename(fn);

	bo = g_malloc((size_t)sizeof(book));
	
	if ( all_bases == NULL ) {
		all_bases = g_list_alloc();
		bo->shown = TRUE;
		bo->deleted = FALSE;
		bo->filename = fn;
		all_bases->data = bo;
		nbook = 1;
	} else {
		bo->shown = TRUE;
		bo->deleted = FALSE;
		bo->filename = fn;
		all_bases = g_list_append(all_bases, (gpointer)bo );
		nbook++;
	}
	
	if ( stat(fn->str, &m) == 0 && ! S_ISREG(m.st_mode) ) {
		st = g_string_new("");
		g_string_sprintf(st, _("You should select a regular file. (%s isn't one)\n"),fn->str);
		g_print(st->str);
		g_string_free(st, 1);
		fflush(NULL);
		return FALSE;
	}
	
	fi = fopen(fn->str, "r" );
	if ( fi == NULL ) { /* oh no, an error ! */
		st = g_string_new(fn->str);
		st = g_string_prepend(st, _("Error at the opening of "));
		st = g_string_append(st, "\n");
		g_print(st->str);
		g_string_free(st, 1);
		fflush(NULL);
		return FALSE;
	}
	
	fseek ( fi, 0, SEEK_END );
	eof = ftell(fi);
	rewind(fi);
	cfi = g_malloc((size_t)(eof+1));
	fread ( cfi, (size_t)1, (size_t)(eof), fi);
	rewind(fi);
	fclose(fi);
	cur_pos = 0;
	st = g_string_new("");
	
	while ( cur_pos != eof ) {
		st = g_string_assign(st,"");
		do {
			c[0] = cfi[cur_pos];
			st = g_string_append_c(st, c[0] );
			cur_pos++;
		} while ( (c[0] != '\n') && (cur_pos!=(eof)) );
		record_add(st, nbook);
	}
	g_free(cfi);
	g_string_free(st, 1);
	return TRUE;
}

gboolean record_remove (gint shown_record)
{
	record *ent;

	if ( shown_record == -2 )
		return FALSE;
	if ( all_records == NULL )
		return FALSE;
	
	record_goto(shown_record);
	if ( all_records->next == NULL ) { /* last record */
		if ( all_records->prev == NULL ) { /* and first record */
			ent = all_records->data;
			g_list_free(all_records);
			g_free(ent);
		} else {
			ent = all_records->data;
			all_records = g_list_previous(all_records);
			g_free(ent);
			all_records->next = NULL;
		}
	} else {
		ent = all_records->data;
		all_records = g_list_remove (all_records, ent);
		g_free(ent);
	}
	return TRUE;
}

gboolean base_visibility ( gint nbk, gboolean isshown )
{
	book *bo;
	gint i;
	
	if ( all_bases == NULL ) { return TRUE; }
	
	all_bases = g_list_first(all_bases);
	
	for (i=1;i<nbk;i++) {
		all_bases = g_list_next(all_bases);
	}
	
	bo = all_bases->data;
	bo->shown = isshown;

	return TRUE;
}

gboolean get_string(record *ent, GString *st)
{
	int i, j;
	GString *tmp;
	tmp = g_string_new("");
	for ( i=0;i<nlabels+1;i++ ) {
		tmp = g_string_assign(tmp, ent->info[i]->str);
		for ( j=0; j<tmp->len;j++ ) {
			switch ( tmp->str[j] ) {
				case '\\' :
					st = g_string_append(st,"\\\\");break;
				case '\n' :
					st = g_string_append(st, "\\n");break;
				case '\r' :
					st = g_string_append(st, "\\r");break;
				case '\t' :
					st = g_string_append(st, "\\t");break;
				case ';' :
					st = g_string_append(st, "\\;");break;
				default :
					st = g_string_append_c(st, tmp->str[j]);
			}
		}
		st = g_string_append_c(st,';');
	}
	st = g_string_erase(st, (st->len)-1, 1);
	st = g_string_append_c(st, '\n');
	g_string_free(tmp, 1);
	return TRUE;
}

gboolean base_write ( gint nbk )
{
	FILE *fi;
	gboolean dontstop=TRUE;
	GString *st;
	record *ent;
	book *bo;
	int i;
	if ( all_bases == NULL ) { return TRUE; }
	if ( all_records == NULL ) { return TRUE; }
	all_bases = g_list_first(all_bases);
	for (i=1;i<nbk;i++) {
		all_bases = g_list_next(all_bases);
	}
	bo = all_bases->data;
	st = g_string_new(bo->filename->str);
	fi = fopen ( st->str, "w" );
	if ( fi == NULL ) { /* oh no, an error ! */
		st = g_string_prepend(st, _("Error at the opening of "));
		st = g_string_append(st, "\n");
		g_print(st->str);
		g_string_free(st, 1);
		fflush(NULL);
		return FALSE;
	}
	
	fputs(_("# You can edit this file by hand but any comment will be destroyed when saved in Gaby\n"), fi);
	all_records = g_list_first(all_records);
	
	if ( all_records->data == NULL ) dontstop = FALSE;
	
	while ( dontstop ) {
		ent = all_records->data;
		if ( ent->frombook == nbk ) {
			st = g_string_assign(st, "");
			get_string(ent, st);
			fputs(st->str, fi);
		}
		
		if ( all_records->next == NULL ) {
			dontstop = FALSE;
		} else {
			all_records = g_list_next(all_records);
		}
	}
	fclose(fi);
	g_string_free(st, 1);
	return TRUE;
}

gboolean base_remove ( gint nbk )
{
	book *bo;
	int i;
	
	if ( all_bases == NULL ) { return TRUE; }

	all_bases = g_list_first(all_bases);
	
	for (i=1;i<nbk;i++) {
		all_bases = g_list_next(all_bases);
	}

	bo = all_bases->data;
	bo->deleted = TRUE;

	return TRUE;
}

void parse_string(GString *st)
{
	int i;
	for (i=0;i<st->len;i++) {
		switch ( st->str[i] ) {
			case '': case '': st->str[i] = 'a'; break;
			case '': case '':
			case '': case '': st->str[i] = 'e'; break;
			case '': case '': st->str[i] = 'i'; break;
			case '':           st->str[i] = 'n'; break;
			case '':           st->str[i] = 'o'; break;
			case '':           st->str[i] = 'u'; break;
		}
	}
}

gint record_select(GString *what, gint ncol, gint type)
{
/* a few words on type : 
 *   0 : watch every records and put all corresponding to shown=TRUE, no
 *       changes to others
 *   1 : same but set shown=FALSE to others
 *   2 : same as 0 but only watch visible records
 *   3 : same as 1 but same as 2
 */
	gboolean dontstop=TRUE;
	record *ent;
	GString *st;
	gint len;
	gint foo;
	gint *az=NULL;
	
	if ( all_records == NULL ) return 1;
	
	all_records = g_list_first(all_records);
	len = what->len;
	parse_string(what);
	
	az = bases_vsblt(az);
	
	do {
		ent = all_records->data;
		if ( type < 2 || ent->shown == TRUE ) {
			st = g_string_new(ent->info[ncol]->str);
			parse_string(st);
			foo = strncasecmp((char*)(what->str), \
					(char*)(st->str), len );
			g_string_free(st, 1);
			if ( foo == 0 && az[ent->frombook - 1 ] == 1 ) {
				ent->shown = TRUE;
			} else {
				if ( type == 1 || type == 3 )ent->shown = FALSE;
			}
		}
		if ( all_records->next == NULL ) {
			dontstop = FALSE;
		} else {
			all_records = g_list_next(all_records);
		}
	} while ( dontstop );
	
	g_free(az);

	return 0;
}

#ifdef TESTBASE

gboolean printbook ()
{
	record *ent;
	GString *st;
	gboolean dontstop=TRUE;
	int i,j=0;
	char sp[2] = { '.', ' ' };
	if ( all_records == NULL ) { return TRUE; }
	all_records = g_list_first(all_records);
	do {
		ent = all_records->data;
		if ( ent->shown == TRUE ) {
			st = g_string_new(ent->info[0]->str);
			st = g_string_append(st, " ");
			st = g_string_append(st, ent->info[1]->str);
/*			st = g_string_append(st, " ");
			for (i=(st->len);i<55;i++) \
				{ st = g_string_append_c(st, sp[j%2] ); }
			st = g_string_append(st, " ");
			st = g_string_append(st, ent->info[9]->str );
*/			st = g_string_append(st, "\n");
			printf(st->str);
			j++;
		}
		if ( all_records->next == NULL ) {
			dontstop = FALSE;
		} else {
			all_records = g_list_next(all_records);
		}
	} while ( dontstop );
	fflush(NULL);
	g_string_free(st, 1);
	return TRUE;
}

int main()
{
	GString *st;
	base_init();
	base_add("~/.gaby/book.ok2");
	nlabels = 16;
	st = g_string_new("");

/* add test */
/*	st = g_string_assign(st, "John Paul;Lepers;;;;Lige;Belgique;;0800/463.41;phone;fax;email;www;1;\n");
	record_add(st,1);
 */

/* search test */
/*	st = g_string_assign(st, "P" );
	record_select(st, 1);

	st = g_string_assign(st, "" );
	record_select(st, 0);
*/
/* sort test */
	base_sort(1);

	printbook();

/* write test */
/*	base_write(1);
*/	
	g_string_free(st, 1);
	return 0;
}
#endif


