/*  Gaby
 *  Copyright (C) 1998 Frederic Peters
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gaby.h"
#include <gtk/gtk.h>
#include "list.h"
#include "form.h"
#include "main.h"
#include "menu_callbacks.h"
#include "dialogs.h"
#include "useful.h"

static GtkWidget *advsearch_window;

static void close_clicked (GtkObject *object, gpointer user_data)
{
	gtk_widget_destroy(advsearch_window);
	advsearch_window = NULL;
}

GtkWidget* create_compar_menu (gint dec, GtkSignalFunc func)
{
	GtkWidget *glade_menuitem;
	GtkWidget *menu;
	gint i;
	
	gchar *items[] = {
		gettext_noop("is less than"),
		gettext_noop("is"),
		gettext_noop("is not"),
		gettext_noop("is greater than")
	};
		
	menu = gtk_menu_new();

	for ( i=0;i<sizeof(items)/sizeof(items[0]);i++ ) {
		glade_menuitem = gtk_menu_item_new_with_label(_(items[i]));
		gtk_signal_connect(GTK_OBJECT(glade_menuitem), "activate", \
				func, (gpointer)(i+dec) );
		gtk_widget_show (glade_menuitem);
		gtk_menu_append (GTK_MENU (menu), glade_menuitem);
	}

	return menu;
}
		
GtkWidget* create_advsearch_window ()
{
	GtkWidget *dialog_vbox;
	GtkWidget *table;
	GtkWidget *andor_menu, *andor_menu_menu, *glade_menuitem;
	GtkWidget *toggle;
	GtkWidget *menu;
	GtkWidget *entry;
	GtkWidget *labels_menu, *compar_menu;
	GtkWidget *labelbox, *label;
	GtkWidget *dialog_action_area;
	GtkWidget *hbuttonbox;
	GtkWidget *searchall;
	GtkWidget *searchshown;
	GtkWidget *close;
	int i;
	
	if ( advsearch_window && advsearch_window->window ) {
		return advsearch_window;
	}

	advsearch_window = gtk_dialog_new ();
	gtk_window_set_title(GTK_WINDOW(advsearch_window),_("Advanced search"));
	gtk_window_set_policy(GTK_WINDOW(advsearch_window), TRUE, TRUE, FALSE);

	dialog_vbox = GTK_DIALOG (advsearch_window)->vbox;
	gtk_widget_show (dialog_vbox);
	gtk_container_border_width (GTK_CONTAINER (dialog_vbox), 5);

	table = gtk_table_new (6, 4, FALSE);
	gtk_widget_show (table);
	gtk_box_pack_start (GTK_BOX (dialog_vbox), table, TRUE, TRUE, 0);
	gtk_container_border_width (GTK_CONTAINER (table), 5);
	gtk_table_set_row_spacings (GTK_TABLE (table), 5);
	gtk_table_set_col_spacings (GTK_TABLE (table), 5);
	
	/* the 'and or or' menu */
	for ( i=0; i<6; i++ ) {
		if ( i == 0 ) {
			andor_menu = gtk_option_menu_new ();
			gtk_widget_show (andor_menu);
			gtk_table_attach (GTK_TABLE (table), andor_menu, 0, 1,\
					0, 1, GTK_EXPAND | GTK_FILL, \
					GTK_EXPAND | GTK_FILL, 0, 0);
			andor_menu_menu = gtk_menu_new ();
			glade_menuitem = gtk_menu_item_new_with_label (_("or"));
			gtk_widget_show (glade_menuitem);
			gtk_menu_append (GTK_MENU (andor_menu_menu), \
					glade_menuitem);
			glade_menuitem = gtk_menu_item_new_with_label(_("and"));
			gtk_widget_show (glade_menuitem);
			gtk_menu_append (GTK_MENU (andor_menu_menu), \
					glade_menuitem);
			gtk_option_menu_set_menu (GTK_OPTION_MENU(andor_menu),\
					andor_menu_menu);
		} else {
			labelbox = gtk_hbox_new(FALSE,0);
			label = gtk_label_new("or");
			gtk_widget_show(label);
			gtk_box_pack_start(GTK_BOX(labelbox),label,TRUE,TRUE,3);
			gtk_widget_show(labelbox);
			toggle = gtk_toggle_button_new();
			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(toggle),\
					FALSE);
			gtk_widget_show(toggle);
			gtk_container_add( GTK_CONTAINER(toggle), labelbox);
			gtk_table_attach (GTK_TABLE (table), toggle, 0, 1,\
					i, i+1, GTK_EXPAND | GTK_FILL, \
					GTK_EXPAND | GTK_FILL, 0, 0);
			
			
		}
		labels_menu = gtk_option_menu_new ();
		gtk_widget_show (labels_menu);
		gtk_table_attach (GTK_TABLE (table), labels_menu, 1, 2, i, i+1,\
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
		menu = create_labels_menu(i*nlabels, GTK_SIGNAL_FUNC(NULL));
		gtk_option_menu_set_menu(GTK_OPTION_MENU(labels_menu), menu);
		
		entry = gtk_entry_new ();
		gtk_widget_show (entry);
		gtk_table_attach (GTK_TABLE (table), entry, 3, 4, i, i+1,
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 2);

		compar_menu = gtk_option_menu_new ();
		gtk_widget_show (compar_menu);
		gtk_table_attach (GTK_TABLE (table), compar_menu, 2, 3, i, i+1,\
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
		menu = create_compar_menu(i*5, GTK_SIGNAL_FUNC(NULL));
		gtk_option_menu_set_menu(GTK_OPTION_MENU(compar_menu), menu);
		
	}
	
	dialog_action_area = GTK_DIALOG (advsearch_window)->action_area;
	gtk_widget_show (dialog_action_area);
	gtk_container_border_width (GTK_CONTAINER (dialog_action_area), 10);

	hbuttonbox = gtk_hbutton_box_new ();
	gtk_widget_show (hbuttonbox);
	gtk_box_pack_start (GTK_BOX (dialog_action_area), hbuttonbox, \
			TRUE, TRUE, 0);

	searchall = gtk_button_new_with_label ( _("Search\nin every\nrecords"));
	gtk_widget_show (searchall);
	gtk_container_add (GTK_CONTAINER (hbuttonbox), searchall);

	searchshown = gtk_button_new_with_label ( \
			_("Search\nonly in\nshown records"));
	gtk_widget_show (searchshown);
	gtk_container_add (GTK_CONTAINER (hbuttonbox), searchshown);

	close = gtk_button_new_with_label (_("Close"));
	gtk_widget_show (close);
	gtk_signal_connect (GTK_OBJECT(close),"clicked", \
			GTK_SIGNAL_FUNC(close_clicked),NULL);
			
	gtk_container_add (GTK_CONTAINER (hbuttonbox), close);
	
	return advsearch_window;
}

