/*  Gaby
 *  Copyright (C) 1998 Frederic Peters
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gaby.h"
#include <gtk/gtk.h>
#include "main.h"
#include "list.h"
#include "form.h"
#include "dialogs.h"
#include "useful.h"

gboolean local_srwc;
gboolean local_ofwos;
gint local_itl[3];
GtkToolbarStyle local_toolbar_type;
gboolean *book_vsblt;
gint local_email_format;

static GtkWidget *config_window;
static GtkCList *clistb;
static GtkWidget *omenu[3];
static GtkWidget *cmd_e_entry;
static GtkWidget *email_fmt;
static GtkWidget *phone_entry[5];
static GtkWidget *print_entry[3];

static void update_vars(gboolean sq);
static void ok_clicked(GtkButton *button, gpointer data);
static void save_clicked(GtkButton *button, gpointer data);
static void apply_clicked(GtkButton *button, gpointer data);
static void close_clicked(GtkButton *button, gpointer data);
static void update_vars(gboolean sq);
static void init_visibility();

void ok_clicked (GtkButton *button, gpointer user_data)
{
	update_vars(FALSE);
	gtk_widget_hide(config_window);
}

void save_clicked (GtkButton *button, gpointer user_data)
{
	update_vars(FALSE);
	save_config_file();
}

void save_config_file()
{
	FILE *fi;
	GString *fn;
	gint i;
	book *bo;
	char tmp[10];
						
	/* save to ~/.gaby/ -sth- rc */

	fn = g_string_new(getenv("HOME"));
	fn = g_string_append(fn, "/.gaby/");
	switch ( prog ) {
		case 1:
		{
			fn = g_string_append(fn, "gabyrc");
		} break;
		case 2:
		{
			fn = g_string_append(fn, "gbcrc");
		} break;
		default:
		{
			fn = g_string_append(fn, name_of_prog->str );
			fn = g_string_append(fn, "rc");
		}
	}
	fi = fopen(fn->str, "w");
	if ( fi == NULL ) {
		g_print(_("Error at the opening of %s\n"), fn->str);
		g_string_free(fn, 1);
		return;
	}
	g_string_free(fn, 1);
	init_visibility();
	
	fputs(_("Gaby preferences file\n"), fi);
	fputs(_("You can change all options within Gaby\n"), fi);
	fputs("\nbegin bases\n", fi);
	all_bases = g_list_first(all_bases);
	for (i=0;i<nbook;i++) {
		bo = all_bases->data;
		if ( bo->deleted == FALSE ) {
			fputs("\t", fi);
			fputs(bo->filename->str, fi);
			fputs(" ;", fi);
			if ( book_vsblt[i] == FALSE ) { fputs(" 0\n", fi); }
			else { fputs(" 1\n", fi); }
		}
		if ( all_bases->next != NULL )
			all_bases = g_list_next(all_bases);
	}
	fputs("end\n", fi);
	
	fputs("\nbegin rows\n", fi);
	for (i=0;i<3;i++) {
		fputs("\t", fi);
		fputs(labels[itl[i]], fi);
		fputs("\n", fi);
	}
	fputs("end\n", fi);

	if ( prog == 1 ) {
		fputs("\nbegin email\n", fi);
		fputs("\tcommand ", fi);
		fputs(cmd_email->str, fi);
		fputs("\n", fi);
		fputs("\tformat ", fi);
		sprintf(tmp, "%d", email_format);
		fputs(tmp, fi);
		fputs("\n", fi);
		fputs("end\n", fi);

		fputs("\nbegin phone\n", fi);
		fputs("\tcommand ", fi);
		fputs(cmd_phone->str, fi);
		fputs("\n\tcountry ", fi);
		fputs(country_code->str, fi);
		fputs("\n\tarea ", fi);
		fputs(area_code->str, fi);
		fputs("\n\tld_prefix ", fi);
		fputs(ld_prefix->str, fi);
		fputs("\n\tin_prefix ", fi);
		fputs(in_prefix->str, fi);
		fputs("\nend\n", fi);
	}
	
	fputs("\nbegin printing\n", fi);
	fputs("\tLaTeX ", fi);
	fputs(latex->str, fi);
	fputs("\n\tdvips ", fi);
	fputs(dvips->str, fi);
	fputs("\n\tGhostView ", fi);
	fputs(ghostview->str, fi);
	fputs("\nend\n", fi);
	
	fputs("\nbegin misc\n", fi);
	fputs("\tsave_record_on_change ", fi);
	if ( srwc == FALSE )  { fputs("no\n", fi); }
	else { fputs("yes\n", fi); }
	fputs("\topen_form_window_on_... ", fi);
	if ( ofwos == FALSE )  { fputs("no\n", fi); }
	else { fputs("yes\n", fi); }
	fputs("\ttoolbar_style ", fi);
	switch ( GTK_TOOLBAR(toolbar)->style ) {
		case GTK_TOOLBAR_ICONS:
			{
				fputs("icons\n", fi);
			} break;
		case GTK_TOOLBAR_TEXT:
			{
				fputs("text\n", fi);
			} break;
		default:
		case GTK_TOOLBAR_BOTH:
			{
				fputs("both\n", fi);
			}
	}
	fputs("end\n\n", fi);
	fclose(fi);
}

void close_clicked (GtkButton *button, gpointer user_data)
{
	gtk_widget_hide(config_window);
}

static gboolean destroy (GtkWidget *widget, gpointer data)
{
	g_free(book_vsblt);
	config_window = create_config_window();
	return TRUE;	
}

void apply_clicked (GtkButton *button, gpointer user_data)
{
	update_vars(FALSE);
}

void fillbookslist ()
{
	book *bo;
	gchar *newrow[1];
	gboolean dontstop=TRUE;
	gint i=0;

	if ( clistb == NULL ) return;
	
	all_bases = g_list_first(all_bases);
	gtk_clist_clear( GTK_CLIST(clistb));
	do {
		bo = all_bases->data;
		if ( bo->deleted == FALSE ) {
			newrow[0] = bo->filename->str;
			gtk_clist_append( GTK_CLIST(clistb), newrow);
			if ( bo->shown == TRUE ) {
				gtk_clist_select_row( GTK_CLIST(clistb), i,0 );
			}
		}
		if ( all_bases->next == NULL ) {
			dontstop = FALSE;
		} else {
			all_bases = g_list_next(all_bases);
		}
		i++;
	} while ( dontstop );

	init_visibility();
}

void init_visibility() {
	book *bo;
	int i;
	g_free(book_vsblt);
	book_vsblt = g_malloc(sizeof(gboolean) * nbook);
	all_bases = g_list_first(all_bases);
	for ( i=0;i<nbook;i++ ) {
		bo = all_bases->data;
		book_vsblt[i] = bo->shown;
		if ( all_bases->next != NULL )
			all_bases = g_list_next(all_bases);
	}
}

void chkbut_toggled(GtkWidget *widget, gint which)
{
	switch ( which ) {
		case 1:
		{
			local_srwc = ! local_srwc;
		} break;
		case 2:
		{
			local_ofwos = ! local_ofwos;
		} break;
	}
}

void update_vars(gboolean sq)
{
/* sq == TRUE => global -> local ; sq == FALSE => local -> global */
	book *bo;
	gint i;
	
	if ( sq ) {
		local_srwc = srwc;
		local_ofwos = ofwos;
		local_itl[0] = itl[0];
		local_itl[1] = itl[1];
		local_itl[2] = itl[2];
		local_toolbar_type = toolbar_type;
		if ( clistb ) {
			fillbookslist(clistb);
		}
		if ( print_entry[0] != NULL ) {
			gtk_entry_set_text(GTK_ENTRY(print_entry[0]), \
					latex->str);
			gtk_entry_set_text(GTK_ENTRY(print_entry[1]), \
					dvips->str);
			gtk_entry_set_text(GTK_ENTRY(print_entry[2]), \
					ghostview->str);
			if ( prog == 1 ) {
				gtk_entry_set_text(GTK_ENTRY(cmd_e_entry), \
					cmd_email->str);
				gtk_entry_set_text(GTK_ENTRY(phone_entry[0]), \
					cmd_phone->str);
				gtk_entry_set_text(GTK_ENTRY(phone_entry[1]), \
					country_code->str);
				gtk_entry_set_text(GTK_ENTRY(phone_entry[2]), \
					area_code->str);
				gtk_entry_set_text(GTK_ENTRY(phone_entry[3]), \
					ld_prefix->str);
				gtk_entry_set_text(GTK_ENTRY(phone_entry[4]), \
					in_prefix->str);
			}
		}
		local_email_format = email_format;
	} else {
		srwc = local_srwc;
		ofwos = local_ofwos;
		itl[0] = local_itl[0];
		itl[1] = local_itl[1];
		itl[2] = local_itl[2];
		latex = g_string_assign(latex, \
				gtk_entry_get_text(GTK_ENTRY(print_entry[0])));
		dvips = g_string_assign(dvips, \
				gtk_entry_get_text(GTK_ENTRY(print_entry[1])));
		ghostview = g_string_assign(ghostview, \
				gtk_entry_get_text(GTK_ENTRY(print_entry[2])));
		if ( prog == 1 ) {
			cmd_email = g_string_assign(cmd_email, \
				gtk_entry_get_text(GTK_ENTRY(cmd_e_entry)));
			cmd_phone = g_string_assign(cmd_phone, \
				gtk_entry_get_text(GTK_ENTRY(phone_entry[0])));
			country_code = g_string_assign(country_code, \
				gtk_entry_get_text(GTK_ENTRY(phone_entry[1])));
			area_code = g_string_assign(area_code, \
				gtk_entry_get_text(GTK_ENTRY(phone_entry[2])));
			ld_prefix = g_string_assign(ld_prefix, \
				gtk_entry_get_text(GTK_ENTRY(phone_entry[3])));
			in_prefix = g_string_assign(in_prefix, \
				gtk_entry_get_text(GTK_ENTRY(phone_entry[4])));
		}
		all_bases = g_list_first(all_bases);
		for ( i=0;i<nbook;i++) {
			bo = all_bases->data;
			bo->shown = book_vsblt[i];
			if ( all_bases->next != NULL )
				all_bases = g_list_next(all_bases);
		}
		email_format = local_email_format;
		show_record_inform();
		fillclist();
		toolbar_type = local_toolbar_type;
		gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), local_toolbar_type);
	}
}

void chg_sel_rowb( GtkWidget *clist, gint row, gint column, \
				GdkEventButton *event, gboolean data)
{
	if ( book_vsblt == NULL ) return;
	book_vsblt[row] = data;
}

void radio_tb(GtkWidget *but, GtkToolbarStyle data)
{
	local_toolbar_type = data;
}

static void label_selected(GtkWidget *wid, gint data)
{
	local_itl[data/(nlabels)] = data%nlabels;
}

void email_format_selected(GtkWidget *wid, gint data)
{
	local_email_format = data;
}

GtkWidget* create_email_fmt_menu()
{
	GtkWidget *glade_menuitem;
	GtkWidget *menu;
	
	menu = gtk_menu_new();
	
	glade_menuitem = gtk_menu_item_new_with_label( \
			"Frederic Peters <fpeters@multimania.com>" );
	gtk_signal_connect(GTK_OBJECT(glade_menuitem), "activate", \
			GTK_SIGNAL_FUNC(email_format_selected), (gpointer)0 );
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (menu), glade_menuitem);
	
	glade_menuitem = gtk_menu_item_new_with_label( \
			"fpeters@multimania.com" );
	gtk_signal_connect(GTK_OBJECT(glade_menuitem), "activate", \
			GTK_SIGNAL_FUNC(email_format_selected), (gpointer)1 );
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (menu), glade_menuitem);
	
	glade_menuitem = gtk_menu_item_new_with_label( \
			"fpeters@multimania.com (Frederic Peters)" );
	gtk_signal_connect(GTK_OBJECT(glade_menuitem), "activate", \
			GTK_SIGNAL_FUNC(email_format_selected), (gpointer)2 );
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (menu), glade_menuitem);
			
	return menu;
}

GtkWidget* create_bases_notebook()
{
	GtkWidget *vboxpage;
	GtkWidget *label;
	
	/* page 'bases' */
	vboxpage = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vboxpage);
	gtk_container_border_width (GTK_CONTAINER (vboxpage), 5);

	clistb = GTK_CLIST(gtk_clist_new (1));
	gtk_widget_show (GTK_WIDGET(clistb));
	gtk_box_pack_start (GTK_BOX (vboxpage),GTK_WIDGET(clistb),TRUE,TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(clistb), "select_row", \
				GTK_SIGNAL_FUNC(chg_sel_rowb), (gpointer)TRUE);
	gtk_signal_connect(GTK_OBJECT(clistb), "unselect_row", \
				GTK_SIGNAL_FUNC(chg_sel_rowb), (gpointer)FALSE);
	
	gtk_container_border_width (GTK_CONTAINER (clistb), 5);
	gtk_clist_set_column_width (GTK_CLIST (clistb), 0, 80);
	gtk_clist_set_selection_mode (GTK_CLIST (clistb), \
						GTK_SELECTION_MULTIPLE);
	gtk_clist_column_titles_show (GTK_CLIST (clistb));

#if ( GTK_MAJOR_VERSION == 1 && \
		GTK_MINOR_VERSION <= 1 && GTK_MICRO_VERSION < 5 )
 	gtk_clist_set_policy (GTK_CLIST (clistb), \
			GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC);
#endif
	fillbookslist();

	label = gtk_label_new (_("Bases"));
	gtk_widget_show (label);
	gtk_clist_set_column_widget (GTK_CLIST (clistb), 0, label);

	label = gtk_label_new (_("NOTE : The selected bases are the shown ones."));
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (vboxpage), label, FALSE, TRUE, 0);

	return vboxpage;
}

GtkWidget* create_misc_notebook()
{
	GtkWidget *vboxpage;
	GtkWidget *chk_button;
	/* Page 'misc' */
	vboxpage = gtk_vbox_new (TRUE, 0);
	gtk_widget_show (vboxpage);
	gtk_container_border_width (GTK_CONTAINER (vboxpage), 5);

	chk_button = gtk_check_button_new_with_label ( \
			_("Save record when going to another one") );
	gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON(chk_button), \
								local_srwc );
	gtk_widget_show (chk_button);
	gtk_box_pack_start (GTK_BOX (vboxpage), chk_button, TRUE, TRUE, 0);
	gtk_signal_connect (GTK_OBJECT (chk_button), "toggled", \
			GTK_SIGNAL_FUNC (chkbut_toggled), (gpointer)1);

	chk_button = gtk_check_button_new_with_label ( \
			_("Open form window when selecting a record in list"));
	gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON(chk_button), \
								local_ofwos );
	gtk_widget_show (chk_button);
	gtk_box_pack_start (GTK_BOX (vboxpage), chk_button, TRUE, TRUE, 0);
	gtk_signal_connect (GTK_OBJECT (chk_button), "toggled", \
			GTK_SIGNAL_FUNC (chkbut_toggled), (gpointer)2);
	
	return vboxpage;
}

GtkWidget* create_toolbar_notebook()
{
	GtkWidget *vboxpage;
	GtkWidget *radio;
	GSList *vboxtb_group = NULL;
	
	/* Page 'toolbar' */
	vboxpage = gtk_vbox_new (TRUE, 0);
	gtk_widget_show (vboxpage);
	gtk_container_border_width (GTK_CONTAINER (vboxpage), 5);

	radio = gtk_radio_button_new_with_label (vboxtb_group, \
			_("Pictures and text"));
	gtk_signal_connect (GTK_OBJECT (radio), "clicked", \
			GTK_SIGNAL_FUNC (radio_tb), (gpointer)GTK_TOOLBAR_BOTH);
	vboxtb_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radio));
	gtk_widget_show (radio);
	gtk_box_pack_start (GTK_BOX (vboxpage), radio, TRUE, TRUE, 0);
	if ( local_toolbar_type == GTK_TOOLBAR_BOTH ) {
		gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (radio), TRUE);
	}

	radio = gtk_radio_button_new_with_label (vboxtb_group, \
			_("Pictures only"));
	gtk_signal_connect (GTK_OBJECT (radio), "clicked", \
			GTK_SIGNAL_FUNC (radio_tb),(gpointer)GTK_TOOLBAR_ICONS);
	vboxtb_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radio));
	gtk_widget_show (radio);
	gtk_box_pack_start (GTK_BOX (vboxpage), radio, TRUE, TRUE, 0);
	if ( local_toolbar_type == GTK_TOOLBAR_ICONS ) {
		gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (radio), TRUE);
	}

	radio = gtk_radio_button_new_with_label (vboxtb_group, \
			_("Text only"));
	gtk_signal_connect (GTK_OBJECT (radio), "clicked", \
			GTK_SIGNAL_FUNC (radio_tb), (gpointer)GTK_TOOLBAR_TEXT);
	vboxtb_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radio));
	gtk_widget_show (radio);
	gtk_box_pack_start (GTK_BOX (vboxpage), radio, TRUE, TRUE, 0);
	if ( local_toolbar_type == GTK_TOOLBAR_TEXT ) {
		gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (radio), TRUE);
	}
	
	return vboxpage;
}

GtkWidget* create_list_notebook()
{
	GtkWidget *vboxpage;
	GtkWidget *label;
	GtkWidget *menu;
	GtkWidget *table;
	
	/* Page 'list' */
	vboxpage = gtk_vbox_new (TRUE, 0);
	gtk_widget_show (vboxpage);
	gtk_container_border_width (GTK_CONTAINER (vboxpage), 5);
	
	table = gtk_table_new (3, 2, FALSE);
	gtk_widget_show (table);
	gtk_box_pack_start (GTK_BOX (vboxpage), table, TRUE, TRUE, 0);
	gtk_table_set_row_spacings (GTK_TABLE (table), 10);
	gtk_table_set_col_spacings (GTK_TABLE (table), 5);
	 
	label = gtk_label_new (_("First column"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

	menu = create_labels_menu(0, GTK_SIGNAL_FUNC(label_selected));
	omenu[0] = gtk_option_menu_new ();
	gtk_widget_show (omenu[0]);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu[0]), menu);
	gtk_option_menu_set_history( GTK_OPTION_MENU(omenu[0]), itl[0] );
	gtk_table_attach (GTK_TABLE (table), omenu[0], 1, 2, 0, 1, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	
	if ( nb_col == 1 ) return vboxpage;
	
	label = gtk_label_new (_("Second column"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	
	menu = create_labels_menu(nlabels, GTK_SIGNAL_FUNC(label_selected));
	omenu[1] = gtk_option_menu_new ();
	gtk_widget_show (omenu[1]);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu[1]), menu);
	gtk_option_menu_set_history( GTK_OPTION_MENU(omenu[1]), itl[1] );
	gtk_table_attach (GTK_TABLE (table), omenu[1], 1, 2, 1, 2, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	
	if ( nb_col == 2 ) return vboxpage;
	
	label = gtk_label_new (_("Third column"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	
	menu = create_labels_menu(nlabels*2, GTK_SIGNAL_FUNC(label_selected));
	omenu[2] = gtk_option_menu_new ();
	gtk_widget_show (omenu[2]);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu[2]), menu);
	gtk_option_menu_set_history( GTK_OPTION_MENU(omenu[2]), itl[2] );
	gtk_table_attach (GTK_TABLE (table), omenu[2], 1, 2, 2, 3, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

	return vboxpage;
}

GtkWidget* create_email_notebook()
{
	GtkWidget *vboxpage;
	GtkWidget *label;
	GtkWidget *table;
	GtkWidget *menu;
	
	/* Page "email" */
	vboxpage = gtk_vbox_new (TRUE, 0);
	gtk_widget_show (vboxpage);
	gtk_container_border_width (GTK_CONTAINER (vboxpage), 5);
	
	table = gtk_table_new (2, 2, FALSE);
	gtk_widget_show (table);
	gtk_box_pack_start (GTK_BOX (vboxpage), table, TRUE, TRUE, 0);
	gtk_table_set_row_spacings (GTK_TABLE (table), 10);
	gtk_table_set_col_spacings (GTK_TABLE (table), 5);

	email_fmt = gtk_option_menu_new ();
	gtk_widget_show (email_fmt);
	gtk_table_attach (GTK_TABLE (table), email_fmt, 1, 2, 1, 2, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	menu = create_email_fmt_menu();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (email_fmt), menu);
	gtk_option_menu_set_history( GTK_OPTION_MENU(email_fmt), \
			local_email_format );
	
	cmd_e_entry = gtk_entry_new_with_max_length (200);
	gtk_entry_set_text(GTK_ENTRY(cmd_e_entry), cmd_email->str );
	gtk_widget_show (cmd_e_entry);
	gtk_table_attach (GTK_TABLE (table), cmd_e_entry, 1, 2, 0, 1, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

	label = gtk_label_new (_("Address format :"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

	label = gtk_label_new (_("Command to execute :"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	
	return vboxpage;
}

GtkWidget* create_phone_notebook()
{
	GtkWidget *vboxpage;
	GtkWidget *label;
	GtkWidget *table;

	/* Page "phone" */
	vboxpage = gtk_vbox_new (TRUE, 0);
	gtk_widget_show (vboxpage);
	gtk_container_border_width (GTK_CONTAINER (vboxpage), 5);

	table = gtk_table_new (5, 2, FALSE);
	gtk_widget_show (table);
	gtk_box_pack_start (GTK_BOX (vboxpage), table, TRUE, TRUE, 0);
	gtk_table_set_row_spacings (GTK_TABLE (table), 10);
	gtk_table_set_col_spacings (GTK_TABLE (table), 5);
	
	label = gtk_label_new (_("Command to execute :"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	phone_entry[0] = gtk_entry_new_with_max_length(200);
	gtk_entry_set_text(GTK_ENTRY(phone_entry[0]), cmd_phone->str );
	gtk_widget_show (phone_entry[0]);
	gtk_table_attach (GTK_TABLE (table), phone_entry[0], 1, 2, 0, 1, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
			
	label = gtk_label_new (_("Country code :"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	phone_entry[1] = gtk_entry_new_with_max_length(10);
	gtk_entry_set_text(GTK_ENTRY(phone_entry[1]), country_code->str );
	gtk_widget_show (phone_entry[1]);
	gtk_table_attach (GTK_TABLE (table), phone_entry[1], 1, 2, 1, 2, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	
	label = gtk_label_new (_("Area code :"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	phone_entry[2] = gtk_entry_new_with_max_length(10);
	gtk_entry_set_text(GTK_ENTRY(phone_entry[2]), area_code->str );
	gtk_widget_show (phone_entry[2]);
	gtk_table_attach (GTK_TABLE (table), phone_entry[2], 1, 2, 2, 3, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	
	label = gtk_label_new (_("Prefix for long distance calls :"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	phone_entry[3] = gtk_entry_new_with_max_length(10);
	gtk_entry_set_text(GTK_ENTRY(phone_entry[3]), ld_prefix->str );
	gtk_widget_show (phone_entry[3]);
	gtk_table_attach (GTK_TABLE (table), phone_entry[3], 1, 2, 3, 4, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	
	label = gtk_label_new (_("Prefix for international calls :"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	phone_entry[4] = gtk_entry_new_with_max_length(4);
	gtk_entry_set_text(GTK_ENTRY(phone_entry[4]), in_prefix->str );
	gtk_widget_show (phone_entry[4]);
	gtk_table_attach (GTK_TABLE (table), phone_entry[4], 1, 2, 4, 5, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

	return vboxpage;
}

GtkWidget* create_print_notebook()
{
	GtkWidget *vboxpage;
	GtkWidget *label;
	GtkWidget *table;

	/* Page "printing" */
	vboxpage = gtk_vbox_new (TRUE, 0);
	gtk_widget_show (vboxpage);
	gtk_container_border_width (GTK_CONTAINER (vboxpage), 5);

	table = gtk_table_new (3, 2, FALSE);
	gtk_widget_show (table);
	gtk_box_pack_start (GTK_BOX (vboxpage), table, TRUE, TRUE, 0);
	gtk_table_set_row_spacings (GTK_TABLE (table), 10);
	gtk_table_set_col_spacings (GTK_TABLE (table), 5);
	
	label = gtk_label_new (_("LaTeX location :"));
	gtk_widget_show(label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	print_entry[0] = gtk_entry_new_with_max_length(200);
	gtk_entry_set_text(GTK_ENTRY(print_entry[0]), latex->str );
	gtk_widget_show (print_entry[0]);
	gtk_table_attach (GTK_TABLE (table), print_entry[0], 1, 2, 0, 1, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	
	label = gtk_label_new (_("dvips location :"));
	gtk_widget_show(label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	print_entry[1] = gtk_entry_new_with_max_length(200);
	gtk_entry_set_text(GTK_ENTRY(print_entry[1]), dvips->str );
	gtk_widget_show (print_entry[1]);
	gtk_table_attach (GTK_TABLE (table), print_entry[1], 1, 2, 1, 2, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

	label = gtk_label_new (_("GhostView location :"));
	gtk_widget_show(label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	print_entry[2] = gtk_entry_new_with_max_length(200);
	gtk_entry_set_text(GTK_ENTRY(print_entry[2]), ghostview->str );
	gtk_widget_show (print_entry[2]);
	gtk_table_attach (GTK_TABLE (table), print_entry[2], 1, 2, 2, 3, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	
	return vboxpage;
}

GtkWidget* create_config_window ()
{
	GtkWidget *bigbox;
	GtkWidget *notebook;
	GtkWidget *vboxpage;
	GtkWidget *label;
	GtkWidget *hs;
	GtkWidget *buttonbox;
	GtkWidget *ok, *apply, *save, *close;

	if ( config_window && config_window->window ) {
		return config_window;
	}

	update_vars(TRUE);
	
	config_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_container_border_width (GTK_CONTAINER (config_window), 5);
	gtk_signal_connect (GTK_OBJECT (config_window), "delete_event", \
			GTK_SIGNAL_FUNC (close_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (config_window), "destroy", \
			GTK_SIGNAL_FUNC (destroy), NULL);
	
	gtk_window_set_title (GTK_WINDOW (config_window), _("Configuration"));
	gtk_window_set_policy (GTK_WINDOW (config_window), TRUE, TRUE, FALSE);

	bigbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (bigbox);
	gtk_container_add (GTK_CONTAINER (config_window), bigbox);

	notebook = gtk_notebook_new ();
	gtk_widget_show (notebook);
	gtk_box_pack_start (GTK_BOX (bigbox), notebook, TRUE, TRUE, 0);
	gtk_container_border_width (GTK_CONTAINER (notebook), 5);

	/* page 'bases' */
	vboxpage = create_bases_notebook();
	label = gtk_label_new(_("Bases"));
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), \
			GTK_WIDGET(vboxpage), GTK_WIDGET(label) );

	/* Page 'misc' */
	vboxpage = create_misc_notebook();
	label = gtk_label_new(_("Misc."));
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), \
			GTK_WIDGET(vboxpage), GTK_WIDGET(label) );

	/* Page 'toolbar' */
	vboxpage = create_toolbar_notebook();
	label = gtk_label_new(_("Toolbar"));
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), \
			GTK_WIDGET(vboxpage), GTK_WIDGET(label) );
	
	/* Page 'list' */
	vboxpage = create_list_notebook();
	label = gtk_label_new(_("List"));
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), \
			GTK_WIDGET(vboxpage), GTK_WIDGET(label) );
	
	/* Page 'printing' */
	vboxpage = create_print_notebook();
	label = gtk_label_new(_("Printing"));
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), \
			GTK_WIDGET(vboxpage), GTK_WIDGET(label) );
	
	if ( prog == 1 ) {
		/* Page "email" */
		vboxpage = create_email_notebook();
		label = gtk_label_new(_("E-Mail"));
		gtk_widget_show(label);
		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), \
				GTK_WIDGET(vboxpage), GTK_WIDGET(label) );
		
		/* Page "phone" */
		vboxpage = create_phone_notebook();
		label = gtk_label_new(_("Phone"));
		gtk_widget_show(label);
		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), \
				GTK_WIDGET(vboxpage), GTK_WIDGET(label) );
	}
	
	/* buttons */
	hs = gtk_hseparator_new ();
	gtk_widget_show (hs);
	gtk_box_pack_start (GTK_BOX (bigbox), hs, FALSE, TRUE, 0);

	buttonbox = gtk_hbutton_box_new ();
	gtk_widget_show (buttonbox);
	gtk_box_pack_start (GTK_BOX (bigbox), buttonbox, FALSE, TRUE, 0);
	gtk_container_border_width (GTK_CONTAINER (buttonbox), 5);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (buttonbox), GTK_BUTTONBOX_SPREAD);

	ok = gtk_button_new_with_label (_("OK"));
	gtk_widget_show (ok);
	gtk_container_add (GTK_CONTAINER (buttonbox), ok);
	gtk_signal_connect (GTK_OBJECT (ok), "clicked", \
			GTK_SIGNAL_FUNC (ok_clicked), 	NULL);

	apply = gtk_button_new_with_label (_("Apply"));
	gtk_widget_show (apply);
	gtk_container_add (GTK_CONTAINER (buttonbox), apply);
	gtk_signal_connect (GTK_OBJECT (apply), "clicked", \
			GTK_SIGNAL_FUNC (apply_clicked), NULL);

	save = gtk_button_new_with_label (_("Save"));
	gtk_widget_show(save);
	gtk_container_add (GTK_CONTAINER (buttonbox), save);
	gtk_signal_connect (GTK_OBJECT (save), "clicked", \
			GTK_SIGNAL_FUNC (save_clicked), NULL);
	
	close = gtk_button_new_with_label (_("Close"));
	gtk_widget_show (close);
	gtk_container_add (GTK_CONTAINER (buttonbox), close);
	gtk_signal_connect (GTK_OBJECT (close), "clicked", \
			GTK_SIGNAL_FUNC (close_clicked), NULL);

	return config_window;
}

