/*  Gaby
 *  Copyright (C) 1998 Frederic Peters
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gaby.h"
#include <gtk/gtk.h>
#include "main.h"
#include "dlg_output.h"
#include "form.h"
#include "useful.h"

#define MAX_COLS_TO_PRINT	6

int print_nbcols;
static char tmpfilename[256];

static GtkWidget *omenu[MAX_COLS_TO_PRINT];
static GtkWidget *table;
static gint col_to_print[MAX_COLS_TO_PRINT];
static GtkWidget *title_entry;

static GtkWidget *win;

void make_latex_output(char *filename)
{
	char *tmp;
	char tempstr[300];
	GString *st;
	record *ent;
	FILE *f;
	gint i;
	gboolean dontstop=TRUE;
	gint *az=NULL;
	
	if ( strlen(filename) < 4 ) {
		sprintf(tempstr, "%s.tex", filename);
	} else {
		if ( memcmp(filename+strlen(filename)-3, "tex", 3) != 0 ) {
			sprintf(tempstr, "%s.tex", filename);
		} else {
			sprintf(tempstr, "%s", filename);
		}
	}
	
	if((f=fopen(tempstr, "w"))==NULL) {
		g_print(_("Unable to create LaTeX file.\n"));
		return;
	}
	
	/* TODO : read this from a file to allow configuration */
	fputs("\\documentclass[10pt,a4paper]{article}\n\n", f);
	fputs("\\usepackage[T1]{fontenc}\n", f);
	tmp=getenv("LANG");
	if ( tmp != NULL && strncmp(tmp, "fr", 2 ) == 0 ) {
		fputs("\\usepackage{french}\n", f);
	}
	fputs("\\usepackage{fullpage}\n", f);
	fputs("\\usepackage{longtable}\n\n", f);
	fputs("\\begin{document}\n\n", f);
	fputs("\\title{", f);
	fputs( gtk_entry_get_text(GTK_ENTRY(title_entry)), f);
	fputs("}\n", f);
	fputs("\\author{}\n", f);
	fputs("\\date{\\today}\n\n", f);
	fputs("\\maketitle\n\n", f);
	sprintf(tempstr,"\\begin{longtable}[c]{|*{%d}{p{%f\\linewidth}|}}\n",\
			print_nbcols, (float)1/print_nbcols*0.85);
	
	fputs(tempstr, f);
	fputs("\\hline\n", f);
	st = g_string_new("");
	for ( i=0; i < print_nbcols; i++ ) {
		st = g_string_append(st, "\\textbf{" );
		st = g_string_append(st, _(labels[col_to_print[i]]) );
		st = g_string_append(st, "}");
		if ( i == print_nbcols-1 ) {
			st = g_string_append(st, " \\endfirsthead\n");
		}else {
			st = g_string_append(st, " & ");
		}
	}
	fputs(st->str, f);
	fputs("\\hline\n", f);
	st = g_string_assign(st, "");
	for ( i=0; i < print_nbcols; i++ ) {
		st = g_string_append(st, "\\textbf{" );
		st = g_string_append(st, _(labels[col_to_print[i]]) );
		st = g_string_append(st, "}");
		if ( i == print_nbcols-1 ) {
			st = g_string_append(st, " \\\\");
		} else {
			st = g_string_append(st, " & ");
		}
	}
	st = g_string_append(st, "\n");
	fputs(st->str, f);
	st = g_string_assign(st, "");
	sprintf(tempstr, "\\textit{%s}", _("(More)") );
	for ( i=0; i < print_nbcols; i++ ) {
		st = g_string_append(st, tempstr );
		if ( i == print_nbcols-1 ) {
			st = g_string_append(st, " \\endhead");
		} else {
			st = g_string_append(st, " & ");
		}
	}
	st = g_string_append(st, "\n");
	fputs(st->str, f);
	sprintf(tempstr, "\\multicolumn{%d}{|c|}{\\textit{%s$\\ldots$}} \\\\ \\hline \\endfoot\n", print_nbcols, _("To be continued") );
	fputs(tempstr, f);
	fputs("\\endlastfoot\n", f);
	fputs("\\hline\n", f);

	/* here comes the records */
	if ( all_records != NULL ) {
		az = bases_vsblt(az);
		all_records = g_list_first(all_records);
		do {
			ent = all_records->data;
			if ( ent->shown == TRUE && az[ent->frombook-1] == TRUE) {
				st = g_string_assign(st, "");
				for ( i=0; i<print_nbcols;i++ ) {
					st = g_string_append(st, \
					  ent->info[col_to_print[i]]->str);
					if ( i == print_nbcols-1 ) {
						st=g_string_append(st,\
							" \\\\ \\hline");
					} else {
						st=g_string_append(st, " & ");
					}
				}
				st = g_string_append(st, "\n");
				fputs(st->str, f);
			}
			if ( all_records->next == NULL ) {
				dontstop = FALSE;
			} else {
				all_records = g_list_next(all_records);
			}
		} while ( dontstop );
	}
	fputs("\\hline\n", f);
	fputs("\\end{longtable}\n\n", f);
	fputs("\\end{document}\n", f);
	
	fclose(f);

	g_string_free(st, 1);
}

void make_html_output(char *filename)
{
	char tempstr[300];
	GString *st;
	record *ent;
	FILE *f;
	gint i;
	gboolean dontstop=TRUE;
	gint *az=NULL;
	
	if ( strlen(filename) < 5 ) {
		sprintf(tempstr, "%s.html", filename);
	} else {
		if ( memcmp(filename+strlen(filename)-4, "html", 4) != 0 ) {
			sprintf(tempstr, "%s.html", filename);
		} else {
			sprintf(tempstr, "%s", filename);
		}
	}
	
	if((f=fopen(tempstr, "w"))==NULL) {
		g_print(_("Unable to create HTML file.\n"));
		return;
	}
	st = g_string_new("");
	/* TODO : read this from a file to allow configuration */
	fputs("<html><head><title>\n", f);
	fputs( gtk_entry_get_text(GTK_ENTRY(title_entry)), f);
	fputs("\n</title></head>\n<body>\n", f);
	fputs("<h1>", f);
	fputs( gtk_entry_get_text(GTK_ENTRY(title_entry)), f);
	fputs("</h1>", f);
	fputs("\n<table border=1>\n", f);
	
	st = g_string_assign(st, "<tr>");
	
	for ( i=0; i < print_nbcols; i++ ) {
		st = g_string_append(st, "<th>" );
		st = g_string_append(st, _(labels[col_to_print[i]]) );
	}
	st = g_string_append(st, "\n");
	fputs(st->str, f);
	
	/* here comes the records */
	if ( all_records != NULL ) {
		az = bases_vsblt(az);
		all_records = g_list_first(all_records);
		do {
			ent = all_records->data;
			if ( ent->shown == TRUE && az[ent->frombook-1]==TRUE) {
				st = g_string_assign(st, "<tr>");
				for ( i=0; i<print_nbcols;i++ ) {
					st = g_string_append(st, "<td>");
					if ( prog == 1 && col_to_print[i] == POS_EMAIL ) {
						g_string_append(st, "<a href=\"mailto:");
						st = g_string_append(st, ent->info[col_to_print[i]]->str);
						g_string_append(st, "\">");
						st = g_string_append(st, ent->info[col_to_print[i]]->str);
						st = g_string_append(st, "</a>");
						continue;
					}
					if ( prog == 1 && col_to_print[i] == POS_WEB ) {
						g_string_append(st, "<a href=\"");
						st = g_string_append(st, ent->info[col_to_print[i]]->str);
						g_string_append(st, "\">");
						st = g_string_append(st, ent->info[col_to_print[i]]->str);
						st = g_string_append(st, "</a>");
						continue;
					}
					st = g_string_append(st, \
					  ent->info[col_to_print[i]]->str);
				}
				st = g_string_append(st, "\n");
				fputs(st->str, f);
			}
			if ( all_records->next == NULL ) {
				dontstop = FALSE;
			} else {
				all_records = g_list_next(all_records);
			}
		} while (dontstop);
	}
	
	fputs("\n</table>\n</body>\n</html>\n", f);
	fclose(f);
}

void make_plaintex_output( char *filename )
{
	char tempstr[300];
	FILE *f;
	
	sprintf(tempstr, "%s.txt", filename);
	if((f=fopen(tempstr, "w"))==NULL) {
		g_print(_("Unable to create file.\n"));
		return;
	}

	fclose(f);
}

static void ok_output_clicked (GtkObject *object, gpointer user_data)
{
	GString *filename;
	gint i;
	
	filename = gtk_object_get_data(GTK_OBJECT(win), "file");
	
	print_nbcols = 0;
	
	for ( i=0; i < MIN(nlabels,MAX_COLS_TO_PRINT); i++) {
		if ( col_to_print[i] != nlabels+1 ) {
			col_to_print[print_nbcols] = col_to_print[i];
			print_nbcols++;
		}
	}

	if ( print_nbcols == 0 ) { return; }

	switch ( (gint)user_data ) {
		case 0: { make_latex_output(filename->str); } break;
		case 1: { make_html_output(filename->str);  } break;
	}
	g_string_free(filename, 1);
	gtk_widget_destroy (win);
	win = NULL;
	message_in_statusbar(_("List created."));
}

static void ok_print_clicked (GtkObject *object, gpointer user_data)
{
	gint i;
	char tempstr[300];
	char *tmp;
	char *wd;
	
	if((tmp=getenv("TMPDIR"))==NULL)
		tmp = (char *)"/tmp";
	wd = malloc(PATH_MAX);
	getcwd(wd, PATH_MAX);
	chdir(tmp);
		
	switch ( prog ) {
		case 1: { sprintf(tmpfilename, "%s/gaby.XXXXXX", tmp );} break;
		case 2: { sprintf(tmpfilename, "%s/gbc.XXXXXX", tmp ); } break;
	}
	
	(void) mktemp (tmpfilename);
	
	print_nbcols = 0;
	
	for ( i=0; i < MIN(nlabels,MAX_COLS_TO_PRINT); i++) {
		if ( col_to_print[i] != nlabels ) {
			col_to_print[print_nbcols] = col_to_print[i];
			print_nbcols++;
		}
	}

	if ( print_nbcols == 0 ) { return; }

	make_latex_output(tmpfilename);

	sprintf(tempstr, "%s %s", latex->str, tmpfilename);
	system(tempstr);
	
	sprintf(tempstr, "%s %s -o", dvips->str, tmpfilename);
	system(tempstr);
	
	gtk_widget_destroy (win);
	win = NULL;
	
	chdir(wd);
	free(wd);
	sprintf(tempstr, "%s %s.ps", ghostview->str, tmpfilename);
	if (fork()) return;
	setsid();
	if(execl("/bin/sh","sh","-c", tempstr, NULL)==-1){exit(100);}
	exit(0);
	
}

static void cancel_clicked (GtkObject *object, gpointer user_data)
{
	gtk_widget_destroy (win);
	win = NULL;
}

static void label_selected(GtkWidget *wid, gint data)
{
	col_to_print[(data)/(nlabels+1)] = (data)%(nlabels+1);
}

gboolean add_a_col()
{
	GtkWidget *label=NULL;
	GtkWidget *glade_menuitem;
	GtkWidget *menu;

	if ( print_nbcols == MAX_COLS_TO_PRINT || print_nbcols == nlabels )
		return FALSE;
	
	print_nbcols++;
	switch ( print_nbcols ) {
		case 1:	{ label = gtk_label_new (_("First column")); } break;
		case 2:	{ label = gtk_label_new (_("Second column"));} break;
		case 3:	{ label = gtk_label_new (_("Third column")); } break;
		case 4:	{ label = gtk_label_new (_("Fourth column"));} break;
		case 5:	{ label = gtk_label_new (_("Fifth column")); } break;
		case 6:	{ label = gtk_label_new (_("Sixth column")); } break;
	}
	
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, print_nbcols, \
			print_nbcols+1,	GTK_EXPAND | GTK_FILL, \
			GTK_EXPAND | GTK_FILL, 0, 0);

	menu = create_labels_menu((print_nbcols-1)*(nlabels+1), \
					GTK_SIGNAL_FUNC(label_selected));
	glade_menuitem = gtk_menu_item_new_with_label(_("None"));
	gtk_signal_connect(GTK_OBJECT(glade_menuitem), "activate", \
			GTK_SIGNAL_FUNC(label_selected), (gpointer) \
					((print_nbcols)*(nlabels+1)) );
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (menu), glade_menuitem);
	
	omenu[print_nbcols-1] = gtk_option_menu_new ();
	gtk_widget_show (omenu[print_nbcols-1]);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu[print_nbcols-1]), menu);
	
	if ( print_nbcols <= nb_col ) {
		gtk_option_menu_set_history( GTK_OPTION_MENU( \
				omenu[print_nbcols-1]), itl[print_nbcols-1] );
		col_to_print[print_nbcols-1] = itl[print_nbcols-1];
	} else {
		gtk_option_menu_set_history( GTK_OPTION_MENU( \
					omenu[print_nbcols-1]), nlabels+1);
		col_to_print[print_nbcols-1] = nlabels+1;
	}
		
	gtk_table_attach (GTK_TABLE (table), omenu[print_nbcols-1], 1, 2, \
			print_nbcols, 	print_nbcols+1,	GTK_EXPAND | GTK_FILL,\
			GTK_EXPAND | GTK_FILL, 0, 0);
	return TRUE;
}

GtkWidget* create_output_dialog (gint type, gchar *filename)
{
	GtkWidget *dialog_vbox;
	GtkWidget *label;
	GtkWidget *hbuttonbox, *vbox;
	GtkWidget *dialog_action_area;
	GtkWidget *ok, *cancel;
	gint i;
	GString *st;
	
	print_nbcols = 0;

	win = gtk_dialog_new ();
	if ( type == 0 ) {
		gtk_window_set_title (GTK_WINDOW (win), _("Print"));
	} else {
		gtk_window_set_title (GTK_WINDOW (win), _("Type of list"));
		st = g_string_new( filename );
		gtk_object_set_data( GTK_OBJECT(win), "file", st );
	}
	gtk_window_set_policy (GTK_WINDOW (win), TRUE, TRUE, FALSE);

	dialog_vbox = GTK_DIALOG (win)->vbox;
	gtk_widget_show (dialog_vbox);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox);
	gtk_box_pack_start (GTK_BOX (dialog_vbox), vbox, TRUE, TRUE, 0);

	table = gtk_table_new (7, 2, FALSE);
	gtk_widget_show (table);
	gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
	gtk_container_border_width (GTK_CONTAINER (table), 5);
	gtk_table_set_row_spacings(GTK_TABLE(table), 10 );
	gtk_table_set_col_spacings(GTK_TABLE(table), 10 );

	title_entry = gtk_entry_new ();
	label = gtk_label_new (_("Title"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

	gtk_widget_show (title_entry);
	gtk_table_attach (GTK_TABLE (table), title_entry, 1, 2, 0, 1, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

	for ( i=0; i<nlabels;i++ ) { add_a_col(); }

	dialog_action_area = GTK_DIALOG (win)->action_area;
	gtk_widget_show (dialog_action_area);
	gtk_container_border_width (GTK_CONTAINER (dialog_action_area), 10);

	hbuttonbox = gtk_hbutton_box_new ();
	gtk_widget_show (hbuttonbox);
	gtk_box_pack_start (GTK_BOX (dialog_action_area), hbuttonbox, \
				TRUE, TRUE, 0);
	
	if ( type == 0 ) {
		ok = gtk_button_new_with_label (_("Print"));
	} else {
		ok = gtk_button_new_with_label (_("OK"));
	}
	
	gtk_widget_show (ok);
	gtk_container_add (GTK_CONTAINER (hbuttonbox), ok);
	
	if ( type == 0 ) {
		gtk_signal_connect (GTK_OBJECT(ok),"clicked", \
			GTK_SIGNAL_FUNC(ok_print_clicked),NULL);
	} else {
		gtk_signal_connect (GTK_OBJECT(ok),"clicked", \
			GTK_SIGNAL_FUNC(ok_output_clicked), (gpointer)(type-1));
	}
	
	cancel = gtk_button_new_with_label (_("Close"));
	gtk_widget_show (cancel);
	gtk_container_add (GTK_CONTAINER (hbuttonbox), cancel);
	gtk_signal_connect (GTK_OBJECT(cancel),"clicked", \
			GTK_SIGNAL_FUNC(cancel_clicked),NULL);
	return win;
}


