/*  Gaby
 *  Copyright (C) 1998 Frederic Peters
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gaby.h"
#include <gtk/gtk.h>
#include "main.h"
#include "form.h"
#include "list.h"
#include "menu.h"
#include "menu_callbacks.h"
#include "../pixmaps/new.xpm"		/* from geg */
#include "../pixmaps/exit.xpm"		/* from geg */
#include "../pixmaps/remove.xpm"	/* from geg */
#include "../pixmaps/previous.xpm"	/* by me    */
#include "../pixmaps/next.xpm"		/* by me    */
#include "../pixmaps/save.xpm"		/* from ??? */

static GtkWidget *create_pixmap(GtkWidget *widget, gchar **data);
static void setbook(GtkWidget *widget, gint foo);

static GtkWidget *statusbar;           // statusbar

void show_record_inform()
{
	/* */
	record *ent=NULL;
	int i=0;
	gint foo;
	gboolean tb;
	GString *st;
	GtkWidget *om_books;

	if ( list_books_changed ) {
		om_books = create_optionmenu_menu(GTK_SIGNAL_FUNC(setbook));
		gtk_option_menu_remove_menu(GTK_OPTION_MENU(form_omenu));
		gtk_option_menu_set_menu(GTK_OPTION_MENU(form_omenu),om_books);
	}

	for (i=0; i<nlabels+1;i++) {
		if ( ffields[i].widget->window == NULL )
			gtk_widget_realize (ffields[i].widget);
	}
	
	if ( (shown_record == -2) || (all_records == NULL) ) /* new record */
	{
		for (i=0;i<nlabels+1;i++) {
			switch ( ffields[i].type ) {
				case 0:
				{
					gtk_entry_set_text ( GTK_ENTRY( \
						ffields[i].widget), "");
				} break;
				case 1:
				{
					gtk_text_set_point(GTK_TEXT( \
						ffields[i].widget), 0);
					gtk_text_forward_delete(GTK_TEXT( \
						ffields[i].widget), \
						gtk_text_get_length( \
						GTK_TEXT(ffields[i].widget)));
				} break;
				case 2:
				{
					gtk_toggle_button_set_state( \
						GTK_TOGGLE_BUTTON(\
						ffields[i].widget), TRUE );
				} break;
			}
		}
		iwb = 1;
		shown_record = -2;
		return;
	}
	if ( shown_record < 0 ) {
		shown_record = 0;
	}
	
	foo = record_goto(shown_record);
	if ( foo != shown_record ) {
		shown_record = foo;
		message_in_statusbar(_("You are on the last record."));
	}
	
	if ( shown_record == -2 ) { show_record_inform(); return;}
	
	ent = all_records->data;
	for (i=0;i<nlabels+1;i++) {
		st = ent->info[i];
		switch ( ffields[i].type ) {
			case 0:
			{
				gtk_entry_set_text (GTK_ENTRY( \
						ffields[i].widget), st->str);
			} break;
			case 1:
			{
				gtk_text_set_point(GTK_TEXT( \
						ffields[i].widget), 0);
				gtk_text_forward_delete(GTK_TEXT( \
						ffields[i].widget), \
						gtk_text_get_length(GTK_TEXT( \
						ffields[i].widget)) );
				gtk_text_insert(GTK_TEXT( \
						ffields[i].widget), NULL, \
						NULL, NULL, st->str,-1);
			} break;
			case 2:
			{
				tb = strcmp(st->str, _("No"));
				gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(\
						ffields[i].widget), tb);
			} break;
		}
				
	}
	
	iwb = ent->frombook;

	gtk_option_menu_set_history( GTK_OPTION_MENU(form_omenu), iwb-1 );

	return;
}

void message_in_statusbar(gchar *msg)
{
	gtk_statusbar_push ( GTK_STATUSBAR (statusbar), \
			gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), \
				"Statusbar"), \
			msg );
}

static void setbook(GtkWidget *widget, gint foo)
{
	iwb = foo;
}

GtkWidget* create_optionmenu_menu(GtkSignalFunc func)
{
	book *bo;
	GString *st;
	GtkWidget *glade_menuitem;
	GtkWidget *om_books;
	gint i;
	gboolean dontstop=TRUE;

	list_books_changed = FALSE;
	om_books = gtk_menu_new ();
	all_bases = g_list_first(all_bases);
	i=1;
	do {
		bo = all_bases->data;
		st = bo->filename;
		glade_menuitem = gtk_menu_item_new_with_label (st->str);
		if ( bo->deleted == TRUE ) {
			gtk_widget_set_sensitive(glade_menuitem, FALSE);
		}
		gtk_signal_connect(GTK_OBJECT(glade_menuitem), "activate", \
				GTK_SIGNAL_FUNC(func), (gpointer)i );
		i++;
		gtk_widget_show (glade_menuitem);
		gtk_menu_append (GTK_MENU (om_books), glade_menuitem);
		if ( all_bases->next == NULL ) {
			dontstop = FALSE;
		} else {
			all_bases = g_list_next(all_bases);
		}
	} while ( dontstop );
	return om_books;
}

void create_toolbar()
{
	GtkWidget *pm;

	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, toolbar_type);
	
#if ( GTK_MAJOR_VERSION == 1 && \
		GTK_MINOR_VERSION == 1 && GTK_MICRO_VERSION == 3 )
	pm = gtk_button_new();
#else
	gtk_widget_realize(pm); /* a warning during the compilation is better
				 than 5 during execution. (i think) */
#endif
	
	gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), _("New"), \
			_("Create a new record in the book"), \
			NULL, create_pixmap(pm, new_xpm), \
			GTK_SIGNAL_FUNC(change_record), (gpointer)3 );
	gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), _("Save"), \
			_("Save this record in a book"), \
			NULL, create_pixmap(pm, save_xpm),
			GTK_SIGNAL_FUNC(save_record), NULL );
	gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), _("Remove"), \
			_("Remove this record from the book"), \
			NULL, create_pixmap(pm, remove_xpm), \
			GTK_SIGNAL_FUNC(remove_record), NULL );
	gtk_toolbar_append_space (GTK_TOOLBAR(toolbar));
	gtk_toolbar_append_item ( GTK_TOOLBAR(toolbar), _("Previous"), \
			_("Go to the previous record"), \
			NULL, create_pixmap(pm, previous_xpm),
			GTK_SIGNAL_FUNC(change_record), (gpointer)1 );
	gtk_toolbar_append_item ( GTK_TOOLBAR(toolbar), _("Next"), \
			_("Go to the next record"), \
			NULL, create_pixmap(pm, next_xpm),
			GTK_SIGNAL_FUNC(change_record), (gpointer)2 );
	gtk_toolbar_append_space (GTK_TOOLBAR(toolbar));
	gtk_toolbar_append_item ( GTK_TOOLBAR(toolbar), _("Exit"), \
			_("Exit Gaby"), \
			NULL, create_pixmap(pm, exit_xpm), \
			GTK_SIGNAL_FUNC(gtk_main_quit), NULL );
}

void chg_toggle (GtkWidget *widget, gpointer data)
{
	GtkWidget *label;
	label = gtk_object_get_data(GTK_OBJECT(widget), "label");
	if (GTK_TOGGLE_BUTTON (widget)->active)
	{
		gtk_label_set(GTK_LABEL(label), _("Yes"));
	} else {
		gtk_label_set(GTK_LABEL(label), _("No"));
	}
}

GtkWidget* create_form_window ()
{
	GtkWidget *win;
	GtkWidget *bigbox;
	GtkWidget *menubar;
	GtkWidget *sclwin;
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *om_books;
	GtkWidget *labelbox;
	gint i;
	gint context_id;
	
	if ( form_window != NULL )
		return form_window;

	win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_container_border_width (GTK_CONTAINER (win), 0);
	gtk_signal_connect (GTK_OBJECT (win), "delete_event", \
			GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
	gtk_signal_connect (GTK_OBJECT (win), "destroy", \
			GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
	gtk_window_set_title (GTK_WINDOW (win), _("Gaby [Form]") );
	gtk_window_set_policy (GTK_WINDOW (win), TRUE, TRUE, FALSE);
	gtk_widget_set_usize (GTK_WIDGET(win), 0, 400);

	bigbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (bigbox);
	gtk_container_add (GTK_CONTAINER (win), bigbox);

	menubar = create_main_menu();
	gtk_box_pack_start(GTK_BOX(bigbox), menubar, FALSE, TRUE, 0);
	gtk_widget_show(menubar);
			
	create_toolbar();
	
	gtk_widget_show (toolbar);
	gtk_box_pack_start (GTK_BOX (bigbox), toolbar,FALSE, TRUE, 1);

	sclwin = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (sclwin);
	gtk_box_pack_start (GTK_BOX (bigbox), sclwin, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW (sclwin), \
			GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	
	table = gtk_table_new ( nlabels+2, 2, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), 5);
	gtk_table_set_col_spacings (GTK_TABLE (table), 5);
	
	for ( i=0;i<nlabels+1;i++ ) {
		label = gtk_label_new ( _(labels[i]) );
		gtk_table_attach (GTK_TABLE (table), label, 1, 2, i+1, i+2, \
				GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, \
				0, 0);
		gtk_widget_show(label);
		switch ( ffields[i].type ) {
			case 0:
			{
				ffields[i].widget = gtk_entry_new();
				gtk_entry_set_editable(GTK_ENTRY( \
						ffields[i].widget), TRUE);
			} break;
			case 1:
			{
				ffields[i].widget = gtk_text_new(NULL, NULL);
				gtk_text_set_editable(GTK_TEXT( \
						ffields[i].widget), TRUE);
				gtk_text_set_word_wrap(GTK_TEXT( \
						ffields[i].widget), TRUE);
			} break;
			case 2:
			{
				labelbox = gtk_hbox_new(FALSE,0);
				label = gtk_label_new("");
				gtk_widget_show(label);
				gtk_box_pack_start(GTK_BOX(labelbox), label, \
						TRUE, TRUE, 3 );
				gtk_widget_show(labelbox);
				ffields[i].widget = gtk_toggle_button_new();
				gtk_object_set_data( GTK_OBJECT( \
						ffields[i].widget), "label", \
						label );
				gtk_signal_connect( GTK_OBJECT( \
						ffields[i].widget), "clicked",\
						GTK_SIGNAL_FUNC(chg_toggle), \
						NULL );
				gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(\
						ffields[i].widget), TRUE );
				gtk_container_add( GTK_CONTAINER( \
						ffields[i].widget), labelbox);
			}
		}
		gtk_table_attach (GTK_TABLE(table), ffields[i].widget, 2, 3, \
				i+1, i+2, GTK_EXPAND | GTK_FILL, GTK_EXPAND | \
				GTK_FILL, 1, 1);
		gtk_widget_show(ffields[i].widget);
	}

	form_omenu = gtk_option_menu_new ();
	gtk_table_attach (GTK_TABLE(table), form_omenu, 2, 3, i+1, i+2, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 5);
	
	gtk_widget_show (form_omenu);
	
	om_books = create_optionmenu_menu(GTK_SIGNAL_FUNC(setbook));
	gtk_option_menu_set_menu(GTK_OPTION_MENU(form_omenu), om_books);

	label = gtk_label_new ( _("In base") );
	gtk_table_attach (GTK_TABLE (table), label, 1, 2, i+1, i+2, \
			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	gtk_widget_show(label);
	
	gtk_widget_show (table);
#if ( GTK_MAJOR_VERSION == 1 && \
		GTK_MINOR_VERSION == 1 && GTK_MICRO_VERSION >= 5 )
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sclwin), \
			table);
#else
	gtk_container_add (GTK_CONTAINER (sclwin), table);
#endif

	statusbar = gtk_statusbar_new ();
	context_id = gtk_statusbar_get_context_id( \
			GTK_STATUSBAR (statusbar), "Statusbar");
			
	gtk_widget_show (statusbar);
	gtk_box_pack_start (GTK_BOX (bigbox), statusbar, FALSE, TRUE, 0);
	
	gtk_statusbar_push ( GTK_STATUSBAR (statusbar), context_id, \
		_("Welcome to Gaby !") );
	
	if ( all_records == NULL ) {
		shown_record = -2;
	} else {
		shown_record = 0;
	}
	
	show_record_inform();
		
	return win;
}



/* create_pixmap, convenience function to create a pixmap widget, from data
 * From geg, by David Bryant (d_bryant@lincoln.college.adelaide.edu.au)
 */

static GtkWidget * create_pixmap(GtkWidget *widget, gchar **data)
{
	GtkStyle *style;
	GdkBitmap *mask;
	GdkPixmap *gdk_pixmap;
	GtkWidget *gtk_pixmap;
	style = gtk_widget_get_style(widget);
	g_assert(style != NULL);
	gdk_pixmap = gdk_pixmap_create_from_xpm_d(widget->window, &mask, \
			&style->bg[GTK_STATE_NORMAL], data);
	g_assert(gdk_pixmap != NULL);
	gtk_pixmap = gtk_pixmap_new(gdk_pixmap, mask);
	g_assert(gtk_pixmap != NULL);
	gtk_widget_show(gtk_pixmap);
	return(gtk_pixmap);
}

