/*  Gaby
 *  Copyright (C) 1998 Frederic Peters
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gaby.h"
#include <gtk/gtk.h>
#include "main.h"
#include "menu.h"
#include "form.h"
#include "list.h"

static gchar* gaby_labels[] =
{
	gettext_noop("First Name"),
	gettext_noop("Last Name"),
#ifndef WITH_COMPANY_NAME
	gettext_noop("(More...)"),
#elif
	gettext_noop("Company Name"),
#endif
	gettext_noop("Street"),
	gettext_noop("Zip"),
	gettext_noop("City"),
	gettext_noop("State / Province"),
	gettext_noop("Country"),
	gettext_noop("Birthday"),
	gettext_noop("Private Phone"),
	gettext_noop("Work Phone"),
	gettext_noop("Cell Phone"),
	gettext_noop("Fax"),
	gettext_noop("E-Mail"),
	gettext_noop("Web Site"),
	gettext_noop("Category"),
	gettext_noop("Remarks")
};

static gchar *gbc_labels[] =
{
	gettext_noop("Title"),
	gettext_noop("Author"),
	gettext_noop("Date"),
	gettext_noop("Pages"),
	gettext_noop("Remarks")
};

static gchar *gpilog_labels[] =
{
	gettext_noop("Date"),
	gettext_noop("Time of flight"),
	gettext_noop("Takeoff from"),
	gettext_noop("Landing at"),
	gettext_noop("Glider call sign"),
	gettext_noop("Instructor"),
	gettext_noop("Launching method")
};

gboolean parse_options( int argc, char *argv[])
{
	int c;
#ifdef HAVE_GETOPT_H
	static struct option long_options[] = {
		{"form", 0, 0, 'f'},
		{"list", 0, 0, 'l'},
		{"help", 0, 0, 'h'},
		{0, 0, 0, 0}
	};
#endif

	startup_mode = 0;
	while(1) {
#ifdef HAVE_GETOPT_H
		c = getopt_long (argc, argv, "flh", long_options, NULL);
#else
		c = getopt (argc, argv, "flh" );
#endif
		if ( c == -1 ) { break; }       /* no more params */
		switch (c) {
			case 'l':
				{
					startup_mode |= 0x02;
				} break;
			case 'f':
				{
					startup_mode |= 0x01;
				} break;
			case 'h':
				{
				g_print(_("\nCommand line options :\n"));	
				g_print(_("  -f, --form : start with the form window\n"));
				g_print(_("  -l, --list : start with the list window\n"));
				g_print(_("  -h, --help : undocumented feature :)\n"));
				g_print("\n");
				return FALSE;
				} break;
		}
	}
	return TRUE;
}

void remove_spaces ( GString *st )
{
	while ( st->str[0] == ' ' || st->str[0] == '\t' ) {
		st = g_string_erase(st, 0, 1 );
	}
	while ( st->str[st->len-1] == ' ' || st->str[st->len-1] == '\t' \
			|| st->str[st->len-1] == '\n' ) {
		st = g_string_erase(st, st->len-1, 1);
	}
}

gint find_separ ( GString *st )
{
	gint i=0;
	while ( st->str[i] != ' ' && st->str[i] != '\t' && i < st->len ) {
		i++;
	}
	if ( i == st->len ) { return -1; }
	return i;
}

gboolean parse_config ( gchar *fna )
{
	FILE *fi;
	long eof, cur_pos;
	char *cfi;
	char c[1];
	GString *st;
	GString *eos;
	GString *fn;
	int sec=0;
	gint nbk=0;
	int i;
	gboolean shown;
	
	fn = g_string_new(fna);
	
	/* loading config file in memory ... */
	switch ( prog ) {
		case 1:
		{
			fn = g_string_append(fn, "/gabyrc" );
		} break;
		case 2:
		{
			fn = g_string_append(fn, "/gbcrc" );
		} break;
		default:
		{
			fn = g_string_append(fn, "/" );
			fn = g_string_append(fn, name_of_prog->str);
			fn = g_string_append(fn, "rc");
		} break;
	}
	fi = fopen ( fn->str, "r" );
	if ( fi == NULL ) {
		/* This is so annoying :) */
		
		/* 
		g_print(_("Couldn't open config file : "));
		g_print(fn->str); g_print("\n");
		*/
		return FALSE;
	}
	fseek ( fi, 0, SEEK_END );
	eof = ftell(fi);
	rewind(fi);
	cfi = g_malloc((size_t)(eof+1));
	fread ( cfi, (size_t)1, (size_t)(eof), fi);
	fclose(fi);

	/* ... and analyzing it */

	st = g_string_new("");
	eos = g_string_new("");
	cur_pos = 0;
	while ( cur_pos != eof ) {
		st = g_string_assign(st,"");
		do {
			c[0] = cfi[cur_pos];
			st = g_string_append_c(st, c[0] );
			cur_pos++;
		} while ( (c[0] != '\n') && (cur_pos!=(eof)) );
		
		remove_spaces(st);

		if ( st->len == 0 || st->str[0] == '#' )
			continue;

		if ( (strncmp(st->str, "begin", (size_t)5)==0) && (sec==0) ) {
			st = g_string_erase(st, 0, 6 );
			if ( strncmp(st->str, "bases", 5) == 0 ) sec = 1;
			if ( strncmp(st->str, "misc", 4) == 0 )  sec = 2;
			if ( strncmp(st->str, "rows", 4) == 0 )  sec = 3;
			switch ( prog  ) {
				case 1:
				{
				if(strncmp(st->str, "email", 5) == 0 ) sec = 4;
				if(strncmp(st->str, "phone", 5) == 0 ) sec = 5;
				} break;
				default: break;
			}
			if ( strncmp(st->str, "printing", 8) == 0 )  sec = 6;
			continue;
		}

		if ( (strncmp(st->str, "end", (size_t)3)==0) && (sec!=0) ) {
			nbk = 0;
			sec = 0;
			continue;
		}
		switch ( sec ) {
			case 1:
			{ /* section 'bases' */
				if ( st->str[st->len-1] == '0' ) {
					shown = FALSE;
				} else {
					shown = TRUE;
				}
				while ( st->str[st->len-1] != ';' ) {
					st = g_string_erase(st, st->len-1, 1);
				}
				st = g_string_erase(st, st->len-1, 1);
				remove_spaces(st);
				base_add(st->str);
				base_visibility(nbk+1, shown);
				nbk++;
			} break;
			case 2:
			{ /* section 'misc' */
				eos = g_string_assign(eos, st->str);
				eos = g_string_erase(eos, 0, find_separ(eos) );
				remove_spaces(eos);
				if ((strncmp(st->str, "save_record_on_change",\
							21) == 0 ) && \
					( strncmp(eos->str, "no", 2) == 0 || \
					  strncmp(eos->str, "0", 1) == 0 ) ) {
					srwc = FALSE;
					continue;
				}
				if ((strncmp(st->str, \
					"open_form_window_on_...",23) == 0 )&&\
					( strncmp(eos->str, "no", 2) == 0 ) ) {
					ofwos = FALSE;
				}
				if (strncmp(st->str,"toolbar_style", 13) == 0){
					if (strncmp(eos->str,"both",4) == 0 ){
						toolbar_type=GTK_TOOLBAR_BOTH;
						continue;
					}
					if (strncmp(eos->str,"icons",5) == 0 ){
						toolbar_type=GTK_TOOLBAR_ICONS;
						continue;
					}
					if (strncmp(eos->str,"text",4) == 0 ){
						toolbar_type=GTK_TOOLBAR_TEXT;
						continue;
					}
				}
			} break;
			case 3:
			{ /* section rows */
				if ( isdigit(st->str[0] ) ) { /* old config */
					sscanf(st->str, "%d", &itl[nbk]);
				} else { /* new config */
					for (i=0;i<=nlabels;i++) {
						if ( strcmp(st->str, \
							labels[i] ) == 0 ) {
							
							itl[nbk] = i;
							break;	
						}
					}
				}
				nbk++;
			} break;
			case 4:
			{ /* section email */
				if ( strncmp(st->str, "command", 7) == 0 ) {
					st = g_string_erase(st, 0, 7);
					remove_spaces(st);
					cmd_email = g_string_assign(cmd_email,\
							st->str);
					continue;
				}
				if ( strncmp(st->str, "format", 6) == 0 ) {
					st = g_string_erase(st, 0, 6);
					remove_spaces(st);
					email_format = atoi(st->str);
					continue;
				}
			} break;
			case 5:
			{ /* section phone */
				if ( strncmp(st->str, "command", 7) == 0 ) {
					st = g_string_erase(st, 0, 7);
					remove_spaces(st);
					cmd_phone = g_string_assign(cmd_phone,\
							st->str);
					continue;
				}
				if ( strncmp(st->str, "country", 7) == 0 ) {
					st = g_string_erase(st, 0, 7);
					remove_spaces(st);
					country_code = g_string_assign( \
							country_code, st->str);
					continue;
				}
				if ( strncmp(st->str, "area", 4) == 0 ) {
					st = g_string_erase(st, 0, 4);
					remove_spaces(st);
					area_code = g_string_assign(area_code,\
							st->str);
					continue;
				}
				if ( strncmp(st->str, "ld_prefix", 9) == 0 ) {
					st = g_string_erase(st, 0, 9);
					remove_spaces(st);
					ld_prefix = g_string_assign(ld_prefix,\
							st->str);
					continue;
				}
				if ( strncmp(st->str, "in_prefix", 9) == 0 ) {
					st = g_string_erase(st, 0, 9);
					remove_spaces(st);
					in_prefix = g_string_assign(in_prefix,\
							st->str);
					continue;
				}
			} break;
			case 6:
			{
				if ( strncmp(st->str, "LaTex", 5) == 0 ) {
					st = g_string_erase(st, 0, 5);
					remove_spaces(st);
					latex = g_string_assign(latex, \
							st->str);
					continue;
				}
				if ( strncmp(st->str, "dvips", 5) == 0 ) {
					st = g_string_erase(st, 0, 5);
					remove_spaces(st);
					dvips = g_string_assign(dvips,\
							st->str);
					continue;
				}
				if ( strncmp(st->str, "GhostView", 9) == 0 ) {
					st = g_string_erase(st, 0, 9);
					remove_spaces(st);
					ghostview = g_string_assign(ghostview,\
							st->str);
					continue;
				}
			} break;
		}
	}
	g_free(cfi);
	g_string_free(st, 1);
	g_string_free(eos, 1);
	return TRUE;
}

gboolean load_user_base()
{
	GString *desc;
	FILE *fi;
	char aline[80];
	int section = -1;
	char *name_field;
	gboolean inafield=FALSE;
	
	desc = g_string_new(getenv("HOME"));
	desc = g_string_append(desc, "/.gaby/desc.");
	desc = g_string_append(desc, name_of_prog->str);
	fi = fopen(desc->str, "r");
	if ( fi == NULL ) {
		desc = g_string_assign(desc, SYSCONF_DIR);
		desc = g_string_assign(desc, "/gaby/desc.");
		desc = g_string_append(desc, name_of_prog->str);
		fi = fopen(desc->str, "r");
		if ( fi == NULL ) return FALSE;
	}
	nlabels = -1;
	
	while ( fgets(aline, 80, fi) != NULL ) {
		if ( strncmp(aline, "begin", 5) == 0 && section == -1 ) {
			if ( memcmp(aline+6, "fields", 6) == 0 ) {
				section = 0;
				continue;
			}
			if ( memcmp(aline+6, "actions", 7) == 0 ) {
				section = 1;
				continue;
			}
		}
		if ( strncmp(aline, "end", 3) == 0 ) {
			section = -1;
			continue;
		}
		memmove(aline, aline+1, strlen(aline+1)+1);
		switch ( section ) {
			case 0:
			{
				if ( inafield == TRUE && aline[0] == '\t') {
					memmove(aline, aline+1, \
							strlen(aline+1)+1);
					switch ( aline[0] ) {
						case 'T':
						{
						memmove(aline, aline+2, \
							strlen(aline+2)+1);
						ffields[nlabels].type = \
								atoi(aline);
						} break;
						case 'F':
						{
						memmove(aline, aline+2, \
							strlen(aline+2)+1);
						if(strstr(aline,"DATE")!=NULL){
						  ffields[nlabels].flags|=DATE;
						  continue;
						}
						if(strstr(aline,"TIME")!=NULL){
						  ffields[nlabels].flags|=TIME;
						  continue;
						}
						if(strstr(aline,"FULLNAME")!=NULL){
						  ffields[nlabels].flags |= FULLNAME;
						  continue;
						}
						
						}
					}
				} else {
					nlabels++;
					name_field = g_malloc(strlen(aline)-1);
					aline[strlen(aline)-1] = 0;
					name_field = strcpy(name_field, aline);
					labels[nlabels] = name_field;
					inafield = TRUE;
				}
			} break;
			case 1:
			{
				if ( inafield == TRUE && aline[0] == '\t') {
					memmove(aline, aline+1, \
							strlen(aline+1)+1);
				} else {
				/* TODO : cammands = g_list_alloc() ... */	
				}
			} break;
			default:
			{
				;
			} break;
		}
				 
	}
	nb_col = ( nlabels > 2 ) ? 3 : 2;
	
	fclose(fi);
	return TRUE;
}

void param_prog()
{
	GString *version;
	int i;
	command *cmd;
	
	version = g_string_new("");
	
	prog = -1;
	
	if ( strcmp(name_of_prog->str, "gaby") == 0 )    { prog = 1; }
	if ( strcmp(name_of_prog->str, "gbc" ) == 0 )    { prog = 2; }
	if ( strcmp(name_of_prog->str, "gpilog" ) == 0 ) { prog = 3; }

	switch ( prog ) {
		case 1:
		{
			nb_col = 3;
			nlabels=(sizeof(gaby_labels)/sizeof(gaby_labels[0]))-1 ;
			for (i=0; i<nlabels+1;i++) {
				labels[i] = gaby_labels[i];
				ffields[i].type = 0;
				ffields[i].flags = 0;
			}
			ffields[8].flags = DATE;
			ffields[nlabels].type = 1;
			version = g_string_assign(version, "1.0");
			cmd = g_malloc(sizeof(command));
			cmd->name = g_string_new(_("Mail to..."));
			cmd->type = 0;
			cmd->cmd = g_string_new("MAILTO");
			cmd->field = POS_EMAIL;
			commands = g_list_alloc();
			commands->data = (gpointer)cmd;
			cmd = g_malloc(sizeof(command));
			cmd->name = g_string_new(_("Phone to..."));
			cmd->cmd = g_string_new("PHONETO");
			cmd->type = 0;
			cmd->field = POS_PHONE;
			commands = g_list_append(commands, (gpointer)cmd);
		} break;
		case 2:
		{
			nb_col = 2;
			nlabels=(sizeof(gbc_labels)/sizeof(gbc_labels[0])) -1 ;
			for (i=0; i<nlabels+1;i++) {
				labels[i] = gbc_labels[i];
				ffields[i].type = 0;
				ffields[i].flags = 0;
			}
			ffields[1].flags = FULLNAME;
			ffields[2].flags = DATE;
			ffields[3].flags = NUMERIC;
			ffields[nlabels].type = 1;
			version = g_string_assign(version, "1.0");
		} break;
		case 3:
		{
			nb_col = 3;
			nlabels=(sizeof(gpilog_labels)/ \
					sizeof(gpilog_labels[0])) -1 ;
			for (i=0; i<nlabels+1;i++) {
				labels[i] = gpilog_labels[i];
				ffields[i].type = 0;
				ffields[i].flags = 0;
			}
			ffields[0].flags = DATE;
			ffields[1].flags = TIME;
			version = g_string_assign(version, "0.1");
		} break;
		case -1:
		{
			if ( load_user_base() == FALSE) {
				g_print(_("Unknown type of base.\n"));
				g_print(_("Exiting\n"));
				exit(1);
			}
		} break;
	}
	
	g_print(_("%s %s using Gaby %s, "), \
				name_of_prog->str, version->str, VERSION );
	switch ( prog ) {
		case 1:
		case 2:
		case 3:
		{
			g_print(_("both copyright (C) 1998 Frederic Peters\n"));
		} break;
		case -1:
		{
			g_print(_("copyright (C) 1998 Frederic Peters\n"));
		} break;
	}
				
	g_string_free(version, 1);

}

int main (int argc, char *argv[])
{
	GString *reptc;
	GString *rcfile;
	gint i;
	
/* looks strange ? gettext.info says :
 * 
 *  setlocale (LC_ALL, "");
 *  bindtextdomain (PACKAGE, LOCALEDIR);
 *  textdomain (PACKAGE);
 * 
 * but so many people had problems with locales that I did it :)
 */
/*
	if ( 
#ifdef HAVE_LC_MESSAGES
		setlocale (LC_MESSAGES, "")
#elif
		setlocale (LC_ALL, "")
#endif
						!= NULL ) {
		bindtextdomain (PACKAGE, LOCALEDIR);
		textdomain (PACKAGE);
		gtk_set_locale ();
	}
*/
/* well ... it's now commented :), back to gettext.info */

	bindtextdomain (PACKAGE, LOCALEDIR);
	textdomain (PACKAGE);
	gtk_set_locale();
		
	reptc = g_string_new("");
	
	/* what kind of base do you want today ? */
	reptc = g_string_assign(reptc, argv[0]);
	i = reptc->len;
	while ( reptc->str[i] != '/' ) { i--; }
	for ( ; i >= 0 ; i-- ) {
		reptc = g_string_erase(reptc, i, 1);
	}

	name_of_prog = g_string_new(reptc->str);
	
	param_prog();
	
	/* Let's create ~/.gaby */
	reptc = g_string_assign (reptc, getenv("HOME"));
	if ( reptc->str[reptc->len-1] != '/' ) {
		reptc = g_string_append_c(reptc, '/' );
	}
	rcfile = g_string_new(reptc->str);
	reptc = g_string_append ( reptc, ".gaby" );
	mkdir(reptc->str, 0755 );

	gtk_init (&argc, &argv);

	gtk_rc_init();
#if ( GTK_MAJOR_VERSION == 1 && \
		GTK_MINOR_VERSION <= 1 && GTK_MICRO_VERSION <= 6 )
	/* Parse gtkrc, I don't know where it should be so
	 * I try a few locations. */
	rcfile = g_string_append(rcfile, ".gaby/gtkrc");
	gtk_rc_parse("/usr/local/share/gtkrc");
	gtk_rc_parse("/usr/share/gtkrc");
	gtk_rc_parse(rcfile->str);
#endif
	/* since gtk 1.1.6 gtk does it itself (but not int ~/.gaby) */
	
	base_init();
	
	/* Parse the command line */
	if ( !parse_options(argc, argv)) {
		exit(1);
	}

	/* Read config file */
	srwc = TRUE;
	ofwos = TRUE;
	toolbar_type = GTK_TOOLBAR_BOTH;
	email_format = 0;
	latex = g_string_new("/usr/bin/latex" );
	dvips = g_string_new("/usr/bin/dvips" );
	ghostview = g_string_new("/usr/X11R6/bin/gv");
	switch ( prog ) {
		case 1:
		{
			itl[0] = 0; itl[1] = 1; itl[2] = POS_PHONE;
			cmd_email = g_string_new("rxvt -bg black -fg white -T Mutt +sb -e mutt \"$e\"");
			cmd_phone = g_string_new("/usr/sbin/chat -l LCK..modem -t 90 ABORT BUSY "" ATZ0 OK ATDP$t OK > /dev/modem < /dev/modem 2>/tmp/err &");
			country_code = g_string_new("");
			area_code = g_string_new("");
			ld_prefix = g_string_new("0");
			in_prefix = g_string_new("00");
		} break;
		case 2:
		{
			itl[0] = 0; itl[1] = 1;
			sort_by_lastname = TRUE;
		} break;
		case 3:
		{
			itl[0] = 0; itl[1] = 2; itl[2] = 3;
		} break;
		case -1:
		{
			for ( i=0;i<nb_col;i++) {
				itl[i] = i;
			}
		} break;
	}
		
	rcfile = g_string_assign(rcfile, SYSCONF_DIR);
	rcfile = g_string_append(rcfile, "/gaby"); 
	parse_config(rcfile->str); /* system wide config */
	g_string_free(rcfile, 1);
	
	parse_config(reptc->str);
	g_string_free(reptc, 1);
	if ( all_bases == NULL ) {
		switch ( prog ) {
			case 1: {
					base_add("~/.gaby/book");
				} break;
			case 2: {
					base_add("~/.gaby/book_gbc");
				} break;
			default:{
					reptc = g_string_new("~/.gaby/book_");
					reptc = g_string_append( reptc, \
							name_of_prog->str);
					base_add(reptc->str);
					g_string_free(reptc, 1);
				} break;
		}
	}

	if ( (startup_mode & 0x01 ) || startup_mode == 0x00 ) {
		startup_mode |= 0x01;
	}

	list_window = create_list_window ();
	if ( startup_mode & 0x02 ) {
		gtk_widget_show (list_window);
	}
	
	form_window = create_form_window ();
	if ( startup_mode & 0x01 ) {
		startup_mode |= 0x01;
		gtk_widget_show (form_window);
	}
	
	gtk_main ();

	return 0;
}

