/*  Gaby
 *  Copyright (C) 1998 Frederic Peters
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gaby.h"
#include <gtk/gtk.h>
#include "menu.h"
#include "menu_callbacks.h"
#include "dlg_config.h"
#include "main.h"

void create_file_menu(GtkWidget *file_me)
{
/* Create the file menu :
 *  - Add or create a base
 *  - Remove a base
 *  - Write a base
 *  - Write all base
 *  - (separator)
 *  - Print
 *  - (separator)
 *  - Preferences
 *  - (separator)
 *  - Quit
 */
	GtkWidget *temp_mi;
	GtkTooltips *tips;
	
	tips = gtk_tooltips_new();
	
	temp_mi = gtk_menu_item_new_with_label(_("Add or create..."));
	gtk_tooltips_set_tip(tips, temp_mi, \
			_("Add an existent base or create a new one"),"");
	gtk_menu_append(GTK_MENU(file_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			(GtkSignalFunc)addbase, NULL);
	gtk_widget_show(temp_mi);

	temp_mi = gtk_menu_item_new_with_label(_("Remove..."));
	gtk_tooltips_set_tip(tips, temp_mi, _("Remove a base"),"");
	gtk_menu_append(GTK_MENU(file_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			(GtkSignalFunc)show_window, (gpointer)4);
	gtk_widget_show(temp_mi);
	
	temp_mi = gtk_menu_item_new_with_label(_("Write..."));
	gtk_tooltips_set_tip(tips, temp_mi, _("Write a base on disk"),"");
	gtk_menu_append(GTK_MENU(file_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			(GtkSignalFunc)show_window, (gpointer)5);
	gtk_widget_show(temp_mi);
					
	temp_mi = gtk_menu_item_new_with_label(_("Write all"));
	gtk_tooltips_set_tip(tips, temp_mi, _("Write every bases on disk"),"");
	gtk_menu_append(GTK_MENU(file_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
				(GtkSignalFunc)write_allbooks, NULL);
	gtk_widget_show(temp_mi);
	
	temp_mi = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(file_me), temp_mi);
	gtk_widget_show(temp_mi);
	
	temp_mi = gtk_menu_item_new_with_label(_("Print..."));
	gtk_tooltips_set_tip(tips, temp_mi, _("Print shown records"),"");
	gtk_menu_append(GTK_MENU(file_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			(GtkSignalFunc)print_records, NULL);
	gtk_widget_show(temp_mi);

	temp_mi = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(file_me), temp_mi);
	gtk_widget_show(temp_mi);

	temp_mi = gtk_menu_item_new_with_label(_("Preferences..."));
	gtk_tooltips_set_tip(tips, temp_mi, _("Set preferences"),"");
	gtk_menu_append(GTK_MENU(file_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			(GtkSignalFunc)show_window, (gpointer)3);
	gtk_widget_show(temp_mi);

	temp_mi = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(file_me), temp_mi);
	gtk_widget_show(temp_mi);
	
	temp_mi = gtk_menu_item_new_with_label(_("Exit"));
	gtk_tooltips_set_tip(tips, temp_mi, _("Exit Gaby"),"");
	gtk_menu_append(GTK_MENU(file_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
				GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
	gtk_widget_show(temp_mi);
	
}

void create_record_menu(GtkWidget *record_me)
{
/* Create the record menu :
 *  - new
 *  - save
 *  - remove
 *  - (separator)
 *  - first
 *  - previous
 *  - next
 *  - last
 *  - (separator)
 *  - goto
 *  - (separator)
 *  (special commands)
 */
	GtkWidget *temp_mi;
	GtkTooltips *tips;
	command *cmd;

	tips = gtk_tooltips_new();
	
	temp_mi = gtk_menu_item_new_with_label(_("New"));
	gtk_tooltips_set_tip(tips, temp_mi, _("Create a new record in a book"),"");
	gtk_menu_append(GTK_MENU(record_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			(GtkSignalFunc)change_record, (gpointer)3);
	gtk_widget_show(temp_mi);

	temp_mi = gtk_menu_item_new_with_label(_("Save"));
	gtk_tooltips_set_tip(tips, temp_mi, _("Save this record in a book"),"");
	gtk_menu_append(GTK_MENU(record_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			GTK_SIGNAL_FUNC(save_record), NULL);
	gtk_widget_show(temp_mi);
	
	temp_mi = gtk_menu_item_new_with_label(_("Remove"));
	gtk_tooltips_set_tip(tips, temp_mi, _("Remove this record from the book"),"");
	gtk_menu_append(GTK_MENU(record_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi),"activate", \
			GTK_SIGNAL_FUNC(remove_record), NULL);
	gtk_widget_show(temp_mi);
				
	temp_mi = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(record_me), temp_mi);
	gtk_widget_show(temp_mi);
			
	temp_mi = gtk_menu_item_new_with_label(_("First"));
	gtk_tooltips_set_tip(tips, temp_mi, _("Go to the first record"),"");
	gtk_menu_append(GTK_MENU(record_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			(GtkSignalFunc)change_record, (gpointer)4);
	gtk_widget_show(temp_mi);

	temp_mi = gtk_menu_item_new_with_label(_("Previous"));
	gtk_tooltips_set_tip(tips, temp_mi, _("Go to the previous record"),"");
	gtk_menu_append(GTK_MENU(record_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			(GtkSignalFunc)change_record, (gpointer)1);
	gtk_widget_show(temp_mi);

	temp_mi = gtk_menu_item_new_with_label(_("Next"));
	gtk_tooltips_set_tip(tips, temp_mi, _("Go to the next record"),"");
	gtk_menu_append(GTK_MENU(record_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			(GtkSignalFunc)change_record, (gpointer)2);
	gtk_widget_show(temp_mi);

	temp_mi = gtk_menu_item_new_with_label(_("Last"));
	gtk_tooltips_set_tip(tips, temp_mi, _("Go to the last record"),"");
	gtk_menu_append(GTK_MENU(record_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			(GtkSignalFunc)change_record, (gpointer)5);
	gtk_widget_show(temp_mi);

	temp_mi = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(record_me), temp_mi);
	gtk_widget_show(temp_mi);
			
	temp_mi = gtk_menu_item_new_with_label(_("Go to..."));
	gtk_tooltips_set_tip(tips, temp_mi, _("Go to a specified record"),"");
	gtk_menu_append(GTK_MENU(record_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			(GtkSignalFunc)change_record, (gpointer)6);
	gtk_widget_show(temp_mi);
	
	if ( commands == NULL ) { return; }
	
	temp_mi = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(record_me), temp_mi);
	gtk_widget_show(temp_mi);

	/*
	if ( prog == 1 ) {
		temp_mi = gtk_menu_item_new_with_label(_("Phone to"));
		gtk_tooltips_set_tip(tips, temp_mi, \
				_("Phone to this person"),"");
		gtk_menu_append(GTK_MENU(record_me), temp_mi);
		gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
				(GtkSignalFunc)action_record, (gpointer)1);
		gtk_widget_show(temp_mi);
	
		temp_mi = gtk_menu_item_new_with_label(_("Mail to"));
		gtk_tooltips_set_tip(tips, temp_mi, \
				_("Mail to this person"),"");
		gtk_menu_append(GTK_MENU(record_me), temp_mi);
		gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
				(GtkSignalFunc)action_record, (gpointer)2);
		gtk_widget_show(temp_mi);
	
		temp_mi = gtk_menu_item_new_with_label(_("Browse to"));
		gtk_tooltips_set_tip(tips, temp_mi, \
				_("Browse to this person web site"),"");
		gtk_menu_append(GTK_MENU(record_me), temp_mi);
		gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
				(GtkSignalFunc)action_record, (gpointer)3);
		gtk_widget_set_sensitive(temp_mi, FALSE);
		gtk_widget_show(temp_mi);
	}
	*/
	
	commands = g_list_first(commands);

	do {
		cmd = commands->data;
		temp_mi = gtk_menu_item_new_with_label(cmd->name->str);
		gtk_menu_append(GTK_MENU(record_me), temp_mi);
		gtk_widget_show(temp_mi);
		if ( cmd->type == 0 ) {
			if ( strcmp(cmd->cmd->str, "MAILTO" ) == 0 ) {
				gtk_signal_connect(GTK_OBJECT(temp_mi), \
					"activate", \
					GTK_SIGNAL_FUNC(action_record), \
					(gpointer) ( cmd->field+128 ));
			} else {
			if ( strcmp(cmd->cmd->str, "PHONETO" ) == 0 ) {
				gtk_signal_connect(GTK_OBJECT(temp_mi), \
					"activate", \
					GTK_SIGNAL_FUNC(action_record), \
					(gpointer) ( cmd->field+64 ));
			}
			}
		}

		if ( commands->next == NULL ) break;
		commands = g_list_next(commands);
	} while(1);
		
}

void create_sort_menu(GtkWidget *sort_me)
{
	GtkWidget *temp_mi;
	GtkTooltips *tips;
	gint i;
	
	tips = gtk_tooltips_new();
	
	for (i=0;i<nlabels+1;i++) {
		temp_mi = gtk_menu_item_new_with_label(_(labels[i]));
		gtk_menu_append(GTK_MENU(sort_me), temp_mi);
		gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
				(GtkSignalFunc)sort_event, (gpointer)i);
		gtk_widget_show(temp_mi);
	}
}

void create_output_menu(GtkWidget *output_me)
{
	GtkWidget *temp_mi;
	//GtkTooltips *tips;

	temp_mi = gtk_menu_item_new_with_label(_("LaTeX..."));
	gtk_menu_append(GTK_MENU(output_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			(GtkSignalFunc)output_list, (gpointer)0);
	gtk_widget_show(temp_mi);
	
	temp_mi = gtk_menu_item_new_with_label(_("HTML..."));
	gtk_menu_append(GTK_MENU(output_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			(GtkSignalFunc)output_list, (gpointer)1);
	gtk_widget_show(temp_mi);
	
}

void create_list_menu(GtkWidget *list_me)
{
/* Create the list menu :
 *  - Show
 *  - (separator)
 *  - Sort
 *  	- ...
 *  	- ...
 *  - Output to
 *      - ...
 *      - ...
 */
	GtkWidget *temp_mi;
	GtkWidget *sort_me, *output_me;
	GtkTooltips *tips;

	tips = gtk_tooltips_new();

	temp_mi = gtk_menu_item_new_with_label(_("Show"));
	gtk_tooltips_set_tip(tips, temp_mi, _("Show the list window"),"");
	gtk_widget_set_sensitive(temp_mi, !(startup_mode & 0x02) );
	gtk_menu_append(GTK_MENU(list_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			(GtkSignalFunc)show_window, (gpointer)2);
	gtk_widget_show(temp_mi);
	gtk_object_set_data(GTK_OBJECT(list_window), "menuitem", temp_mi);
	
	temp_mi = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(list_me), temp_mi);
	gtk_widget_show(temp_mi);
	
	sort_me = gtk_menu_new();
	create_sort_menu(sort_me);
	
	temp_mi = gtk_menu_item_new_with_label(_("Sort by "));
	gtk_tooltips_set_tip(tips, temp_mi, _("Sort the list"),"");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(temp_mi), sort_me);
	gtk_menu_append(GTK_MENU(list_me), temp_mi);
	gtk_widget_show(temp_mi);

	output_me = gtk_menu_new();
	create_output_menu(output_me);
	
	temp_mi = gtk_menu_item_new_with_label(_("Output to "));
	gtk_tooltips_set_tip(tips, temp_mi,_("Create a file from the list"),"");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(temp_mi), output_me);
	gtk_menu_append(GTK_MENU(list_me), temp_mi);
	gtk_widget_show(temp_mi);
	
}

void create_help_menu(GtkWidget *help_me)
{
/* Create the help menu
 *  - Help
 *  - About
 */
	GtkWidget *temp_mi;
	GtkTooltips *tips;

	tips = gtk_tooltips_new();
/*	
	temp_mi = gtk_menu_item_new_with_label(_("Help"));
//	gtk_tooltips_set_tip(tips,temp_mi,_(""),"");
	gtk_menu_append(GTK_MENU(help_me), temp_mi);
	gtk_widget_set_sensitive(temp_mi, FALSE);
//	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
//			GTK_SIGNAL_FUNC(show_window), 0);
	gtk_widget_show(temp_mi);
*/				
	temp_mi = gtk_menu_item_new_with_label(_("About..."));
	gtk_tooltips_set_tip(tips,temp_mi,_("Show various info about Gaby"),"");
	gtk_menu_append(GTK_MENU(help_me), temp_mi);
	gtk_signal_connect(GTK_OBJECT(temp_mi), "activate", \
			GTK_SIGNAL_FUNC(show_window), 0);
	gtk_widget_show(temp_mi);
}

GtkWidget* create_main_menu()
{
	GtkWidget *mb;
	GtkWidget *file_me, *record_me, *list_me, *help_me;
	GtkWidget *temp_mi;
	mb = gtk_menu_bar_new();
	
	file_me = gtk_menu_new();
	create_file_menu(file_me);
	temp_mi = gtk_menu_item_new_with_label(_("Base"));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(temp_mi), file_me);
	gtk_menu_bar_append(GTK_MENU_BAR(mb), temp_mi);
	gtk_widget_show(temp_mi);
	
	record_me = gtk_menu_new();
	create_record_menu(record_me);
	temp_mi = gtk_menu_item_new_with_label(_("Record"));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(temp_mi), record_me);
	gtk_menu_bar_append(GTK_MENU_BAR(mb), temp_mi);
	gtk_widget_show(temp_mi);
	
	list_me = gtk_menu_new();
	create_list_menu(list_me);
	temp_mi = gtk_menu_item_new_with_label(_("List"));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(temp_mi), list_me);
	gtk_menu_bar_append(GTK_MENU_BAR(mb), temp_mi);
	gtk_widget_show(temp_mi);
	
/*	window_me = gtk_menu_new();
	create_window_menu(window_me);
	temp_mi = gtk_menu_item_new_with_label(_("Window"));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(temp_mi), window_me);
	gtk_menu_bar_append(GTK_MENU_BAR(mb), temp_mi);
	gtk_widget_show(temp_mi);
*/	
	help_me = gtk_menu_new();
	create_help_menu(help_me);
	temp_mi = gtk_menu_item_new_with_label(_("Help"));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(temp_mi), help_me);
	gtk_menu_bar_append(GTK_MENU_BAR(mb), temp_mi);
	gtk_menu_item_right_justify(GTK_MENU_ITEM(temp_mi));
	gtk_widget_show(temp_mi);
	
	return mb;
}

